import datetime
import locale
import logging
import os.path
from posixpath import join as posix_join
from sys import platform

from jinja2.utils import generate_lorem_ipsum

from pelican.contents import Article, Author, Category, Page, Static
from pelican.plugins.signals import content_object_init
from pelican.settings import DEFAULT_CONFIG
from pelican.tests.support import LoggedTestCase, get_context, get_settings, unittest
from pelican.utils import path_to_url, posixize_path, truncate_html_words

# generate 3 test paragraphs, each enclosed with <p>
# save the first paragraph separately for testing the summary generation algorithm
# NOTE: these values are nondeterministic between test runs
TEST_CONTENT_FIRST_PARAGRAPH = str(generate_lorem_ipsum(n=1))
TEST_CONTENT_REMAINING_PARAGRAPHS = str(generate_lorem_ipsum(n=2))
TEST_CONTENT = TEST_CONTENT_FIRST_PARAGRAPH + TEST_CONTENT_REMAINING_PARAGRAPHS
TEST_SUMMARY = generate_lorem_ipsum(n=1, html=False)


class TestBase(LoggedTestCase):
    def setUp(self):
        super().setUp()
        self.old_locale = locale.setlocale(locale.LC_ALL)
        locale.setlocale(locale.LC_ALL, "C")
        self.page_kwargs = {
            "content": TEST_CONTENT,
            "context": {
                "localsiteurl": "",
                "generated_content": {},
                "static_content": {},
                "static_links": set(),
            },
            "metadata": {
                "summary": TEST_SUMMARY,
                "title": "foo bar",
                "author": Author("Blogger", DEFAULT_CONFIG),
            },
            "source_path": "/path/to/file/foo.ext",
        }
        self._disable_limit_filter()

    def tearDown(self):
        locale.setlocale(locale.LC_ALL, self.old_locale)
        self._enable_limit_filter()

    def _disable_limit_filter(self):
        from pelican.contents import logger

        logger.disable_filter()

    def _enable_limit_filter(self):
        from pelican.contents import logger

        logger.enable_filter()

    def _copy_page_kwargs(self):
        # make a deep copy of page_kwargs
        page_kwargs = {key: self.page_kwargs[key] for key in self.page_kwargs}
        for key in page_kwargs:
            if not isinstance(page_kwargs[key], dict):
                break
            page_kwargs[key] = {
                subkey: page_kwargs[key][subkey] for subkey in page_kwargs[key]
            }

        return page_kwargs


class TestPage(TestBase):
    def test_use_args(self):
        # Creating a page with arguments passed to the constructor should use
        # them to initialise object's attributes.
        metadata = {
            "foo": "bar",
            "foobar": "baz",
            "title": "foobar",
        }
        page = Page(TEST_CONTENT, metadata=metadata, context={"localsiteurl": ""})
        for key, value in metadata.items():
            self.assertTrue(hasattr(page, key))
            self.assertEqual(value, getattr(page, key))
        self.assertEqual(page.content, TEST_CONTENT)

    def test_mandatory_properties(self):
        # If the title is not set, must throw an exception.
        page = Page("content")
        self.assertFalse(page._has_valid_mandatory_properties())
        self.assertLogCountEqual(
            count=1,
            msg="Skipping .*: could not find information about 'title'",
            level=logging.ERROR,
        )
        page = Page("content", metadata={"title": "foobar"})
        self.assertTrue(page._has_valid_mandatory_properties())

    def test_summary_from_metadata(self):
        # If a :summary: metadata is given, it should be used
        page = Page(**self.page_kwargs)
        self.assertEqual(page.summary, TEST_SUMMARY)

    def test_summary_max_length(self):
        # If a :SUMMARY_MAX_LENGTH: is set, and there is no other summary,
        # generated summary should not exceed the given length.
        page_kwargs = self._copy_page_kwargs()
        settings = get_settings()
        page_kwargs["settings"] = settings
        del page_kwargs["metadata"]["summary"]
        settings["SUMMARY_MAX_LENGTH"] = None
        page = Page(**page_kwargs)
        self.assertEqual(page.summary, TEST_CONTENT)
        settings["SUMMARY_MAX_LENGTH"] = 10
        page = Page(**page_kwargs)
        self.assertEqual(page.summary, truncate_html_words(TEST_CONTENT, 10))
        settings["SUMMARY_MAX_LENGTH"] = 0
        page = Page(**page_kwargs)
        self.assertEqual(page.summary, "")

    def test_summary_paragraph(self):
        # If SUMMARY_MAX_PARAGRAPHS is set, the generated summary should
        # not exceed the given paragraph count.
        page_kwargs = self._copy_page_kwargs()
        settings = get_settings()
        page_kwargs["settings"] = settings
        del page_kwargs["metadata"]["summary"]
        settings["SUMMARY_MAX_PARAGRAPHS"] = 1
        settings["SUMMARY_MAX_LENGTH"] = None
        page = Page(**page_kwargs)
        self.assertEqual(page.summary, TEST_CONTENT_FIRST_PARAGRAPH)

    def test_summary_paragraph_max_length(self):
        # If both SUMMARY_MAX_PARAGRAPHS and SUMMARY_MAX_LENGTH are set,
        # the generated summary should not exceed the given paragraph count and
        # not exceed the given length.
        page_kwargs = self._copy_page_kwargs()
        settings = get_settings()
        page_kwargs["settings"] = settings
        del page_kwargs["metadata"]["summary"]
        settings["SUMMARY_MAX_PARAGRAPHS"] = 1
        settings["SUMMARY_MAX_LENGTH"] = 10
        page = Page(**page_kwargs)
        self.assertEqual(
            page.summary, truncate_html_words(TEST_CONTENT_FIRST_PARAGRAPH, 10)
        )

    def test_summary_paragraph_long_max_length(self):
        # If both SUMMARY_MAX_PARAGRAPHS and SUMMARY_MAX_LENGTH are set,
        # and the first SUMMARY_MAX_PARAGRAPHS paragraphs have fewer words
        # than SUMMARY_MAX_LENGTH, the generated summary should still only
        # contain the given number of paragraphs.
        page_kwargs = self._copy_page_kwargs()
        settings = get_settings()
        page_kwargs["settings"] = settings
        del page_kwargs["metadata"]["summary"]
        settings["SUMMARY_MAX_PARAGRAPHS"] = 1
        settings["SUMMARY_MAX_LENGTH"] = 50
        page = Page(**page_kwargs)
        self.assertEqual(
            page.summary, truncate_html_words(TEST_CONTENT_FIRST_PARAGRAPH, 50)
        )

    def test_summary_end_suffix(self):
        # If a :SUMMARY_END_SUFFIX: is set, and there is no other summary,
        # generated summary should contain the specified marker at the end.
        page_kwargs = self._copy_page_kwargs()
        settings = get_settings()
        page_kwargs["settings"] = settings
        del page_kwargs["metadata"]["summary"]
        settings["SUMMARY_END_SUFFIX"] = "test_marker"
        settings["SUMMARY_MAX_LENGTH"] = 10
        page = Page(**page_kwargs)
        self.assertEqual(
            page.summary, truncate_html_words(TEST_CONTENT, 10, "test_marker")
        )
        self.assertIn("test_marker", page.summary)

    def test_summary_get_summary_warning(self):
        """calling ._get_summary() should issue a warning"""
        page_kwargs = self._copy_page_kwargs()
        page = Page(**page_kwargs)
        self.assertEqual(page.summary, TEST_SUMMARY)
        self.assertEqual(page._get_summary(), TEST_SUMMARY)
        self.assertLogCountEqual(
            count=1,
            msg=r"_get_summary\(\) has been deprecated since 3\.6\.4\. "
            "Use the summary decorator instead",
            level=logging.WARNING,
        )

    def test_slug(self):
        page_kwargs = self._copy_page_kwargs()
        settings = get_settings()
        page_kwargs["settings"] = settings
        settings["SLUGIFY_SOURCE"] = "title"
        page = Page(**page_kwargs)
        self.assertEqual(page.slug, "foo-bar")
        settings["SLUGIFY_SOURCE"] = "basename"
        page = Page(**page_kwargs)
        self.assertEqual(page.slug, "foo")

        # test slug from title with unicode and case

        inputs = (
            # (title, expected, preserve_case, use_unicode)
            ("指導書", "zhi-dao-shu", False, False),
            ("指導書", "Zhi-Dao-Shu", True, False),
            ("指導書", "指導書", False, True),
            ("指導書", "指導書", True, True),
            ("Çığ", "cig", False, False),
            ("Çığ", "Cig", True, False),
            ("Çığ", "çığ", False, True),
            ("Çığ", "Çığ", True, True),
        )

        settings = get_settings()
        page_kwargs = self._copy_page_kwargs()
        page_kwargs["settings"] = settings

        for title, expected, preserve_case, use_unicode in inputs:
            settings["SLUGIFY_PRESERVE_CASE"] = preserve_case
            settings["SLUGIFY_USE_UNICODE"] = use_unicode
            page_kwargs["metadata"]["title"] = title
            page = Page(**page_kwargs)
            self.assertEqual(page.slug, expected, (title, preserve_case, use_unicode))

    def test_defaultlang(self):
        # If no lang is given, default to the default one.
        page = Page(**self.page_kwargs)
        self.assertEqual(page.lang, DEFAULT_CONFIG["DEFAULT_LANG"])

        # it is possible to specify the lang in the metadata infos
        self.page_kwargs["metadata"].update(
            {
                "lang": "fr",
            }
        )
        page = Page(**self.page_kwargs)
        self.assertEqual(page.lang, "fr")

    def test_save_as(self):
        # If a lang is not the default lang, save_as should be set
        # accordingly.

        # if a title is defined, save_as should be set
        page = Page(**self.page_kwargs)
        self.assertEqual(page.save_as, "pages/foo-bar.html")

        # if a language is defined, save_as should include it accordingly
        self.page_kwargs["metadata"].update(
            {
                "lang": "fr",
            }
        )
        page = Page(**self.page_kwargs)
        self.assertEqual(page.save_as, "pages/foo-bar-fr.html")

    def test_relative_source_path(self):
        # 'relative_source_path' should be the relative path
        # from 'PATH' to 'source_path'
        page_kwargs = self._copy_page_kwargs()

        # If 'source_path' is None, 'relative_source_path' should
        # also return None
        page_kwargs["source_path"] = None
        page = Page(**page_kwargs)
        self.assertIsNone(page.relative_source_path)

        page_kwargs = self._copy_page_kwargs()
        settings = get_settings()
        full_path = page_kwargs["source_path"]

        settings["PATH"] = os.path.dirname(full_path)
        page_kwargs["settings"] = settings
        page = Page(**page_kwargs)

        # if 'source_path' is set, 'relative_source_path' should
        # return the relative path from 'PATH' to 'source_path'
        self.assertEqual(
            page.relative_source_path,
            os.path.relpath(full_path, os.path.dirname(full_path)),
        )

    def test_metadata_url_format(self):
        # Arbitrary metadata should be passed through url_format()
        page = Page(**self.page_kwargs)
        self.assertIn("summary", page.url_format.keys())
        page.metadata["directory"] = "test-dir"
        page.settings = get_settings(PAGE_SAVE_AS="{directory}/{slug}")
        self.assertEqual(page.save_as, "test-dir/foo-bar")

    def test_datetime(self):
        # If DATETIME is set to a tuple, it should be used to override LOCALE
        dt = datetime.datetime(2015, 9, 13)

        page_kwargs = self._copy_page_kwargs()

        # set its date to dt
        page_kwargs["metadata"]["date"] = dt
        page = Page(**page_kwargs)

        # page.locale_date is a unicode string in both python2 and python3
        dt_date = dt.strftime(DEFAULT_CONFIG["DEFAULT_DATE_FORMAT"])

        self.assertEqual(page.locale_date, dt_date)
        page_kwargs["settings"] = get_settings()

        # I doubt this can work on all platforms ...
        if platform == "win32":
            locale = "jpn"
        else:
            locale = "ja_JP.utf8"
        page_kwargs["settings"]["DATE_FORMATS"] = {"jp": (locale, "%Y-%m-%d(%a)")}
        page_kwargs["metadata"]["lang"] = "jp"

        import locale as locale_module

        try:
            page = Page(**page_kwargs)
            self.assertEqual(page.locale_date, "2015-09-13(\u65e5)")
        except locale_module.Error:
            # The constructor of ``Page`` will try to set the locale to
            # ``ja_JP.utf8``. But this attempt will failed when there is no
            # such locale in the system. You can see which locales there are
            # in your system with ``locale -a`` command.
            #
            # Until we find some other method to test this functionality, we
            # will simply skip this test.
            unittest.skip(f"There is no locale {locale} in this system.")

    def test_template(self):
        # Pages default to page, metadata overwrites
        default_page = Page(**self.page_kwargs)
        self.assertEqual("page", default_page.template)
        page_kwargs = self._copy_page_kwargs()
        page_kwargs["metadata"]["template"] = "custom"
        custom_page = Page(**page_kwargs)
        self.assertEqual("custom", custom_page.template)

    def test_signal(self):
        def receiver_test_function(sender):
            receiver_test_function.has_been_called = True

        receiver_test_function.has_been_called = False

        content_object_init.connect(receiver_test_function)
        self.assertIn(receiver_test_function, content_object_init.receivers_for(Page))

        self.assertFalse(receiver_test_function.has_been_called)
        Page(**self.page_kwargs)
        self.assertTrue(receiver_test_function.has_been_called)

    def test_get_content(self):
        # Test that the content is updated with the relative links to
        # filenames, tags and categories.
        settings = get_settings()
        args = self.page_kwargs.copy()
        args["settings"] = settings

        # Tag
        args["content"] = 'A simple test, with a <a href="|tag|tagname">link</a>'
        page = Page(**args)
        content = page.get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            (
                "A simple test, with a "
                '<a href="http://notmyidea.org/tag/tagname.html">link</a>'
            ),
        )

        # Category
        args["content"] = 'A simple test, with a <a href="|category|category">link</a>'
        page = Page(**args)
        content = page.get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            (
                "A simple test, with a "
                '<a href="http://notmyidea.org/category/category.html">link</a>'
            ),
        )

    def test_intrasite_link(self):
        cls_name = "_DummyArticle"
        article = type(cls_name, (object,), {"url": "article.html"})

        args = self.page_kwargs.copy()
        args["settings"] = get_settings()
        args["source_path"] = "content"
        args["context"]["generated_content"] = {"article.rst": article}

        # Classic intrasite link via filename
        args["content"] = (
            'A simple test, with a <a href="|filename|article.rst">link</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "A simple test, with a "
            '<a href="http://notmyidea.org/article.html">link</a>',
        )

        # fragment
        args["content"] = (
            "A simple test, with a "
            '<a href="|filename|article.rst#section-2">link</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "A simple test, with a "
            '<a href="http://notmyidea.org/article.html#section-2">link</a>',
        )

        # query
        args["content"] = (
            "A simple test, with a "
            '<a href="|filename|article.rst'
            '?utm_whatever=234&highlight=word">link</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "A simple test, with a "
            '<a href="http://notmyidea.org/article.html'
            '?utm_whatever=234&highlight=word">link</a>',
        )

        # combination
        args["content"] = (
            "A simple test, with a "
            '<a href="|filename|article.rst'
            '?utm_whatever=234&highlight=word#section-2">link</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "A simple test, with a "
            '<a href="http://notmyidea.org/article.html'
            '?utm_whatever=234&highlight=word#section-2">link</a>',
        )

        # also test for summary in metadata
        parsed = (
            'A simple summary test, with a <a href="|filename|article.rst">link</a>'
        )
        linked = (
            "A simple summary test, with a "
            '<a href="http://notmyidea.org/article.html">link</a>'
        )
        args["settings"]["FORMATTED_FIELDS"] = ["summary", "custom"]
        args["metadata"]["summary"] = parsed
        args["metadata"]["custom"] = parsed
        args["context"]["localsiteurl"] = "http://notmyidea.org"
        p = Page(**args)
        # This is called implicitly from all generators and Pelican.run() once
        # all files are processed. Here we process just one page so it needs
        # to be called explicitly.
        p.refresh_metadata_intersite_links()
        self.assertEqual(p.summary, linked)
        self.assertEqual(p.custom, linked)

    def test_intrasite_link_more(self):
        cls_name = "_DummyAsset"

        args = self.page_kwargs.copy()
        args["settings"] = get_settings()
        args["source_path"] = "content"
        args["context"]["static_content"] = {
            "images/poster.jpg": type(
                cls_name, (object,), {"url": "images/poster.jpg"}
            ),
            "assets/video.mp4": type(cls_name, (object,), {"url": "assets/video.mp4"}),
            "images/graph.svg": type(cls_name, (object,), {"url": "images/graph.svg"}),
        }
        args["context"]["generated_content"] = {
            "reference.rst": type(cls_name, (object,), {"url": "reference.html"}),
        }

        # video.poster
        args["content"] = (
            "There is a video with poster "
            '<video controls poster="{static}/images/poster.jpg">'
            '<source src="|static|/assets/video.mp4" type="video/mp4">'
            "</video>"
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "There is a video with poster "
            '<video controls poster="http://notmyidea.org/images/poster.jpg">'
            '<source src="http://notmyidea.org/assets/video.mp4"'
            ' type="video/mp4">'
            "</video>",
        )

        # object.data
        args["content"] = (
            "There is a svg object "
            '<object data="{static}/images/graph.svg"'
            ' type="image/svg+xml">'
            "</object>"
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "There is a svg object "
            '<object data="http://notmyidea.org/images/graph.svg"'
            ' type="image/svg+xml">'
            "</object>",
        )

        # blockquote.cite
        args["content"] = (
            "There is a blockquote with cite attribute "
            '<blockquote cite="{filename}reference.rst">blah blah</blockquote>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "There is a blockquote with cite attribute "
            '<blockquote cite="http://notmyidea.org/reference.html">'
            "blah blah"
            "</blockquote>",
        )

    def test_intrasite_link_absolute(self):
        """Test that absolute URLs are merged properly."""

        args = self.page_kwargs.copy()
        args["settings"] = get_settings(
            STATIC_URL="http://static.cool.site/{path}",
            ARTICLE_URL="http://blog.cool.site/{slug}.html",
        )
        args["source_path"] = "content"
        args["context"]["static_content"] = {
            "images/poster.jpg": Static(
                "", settings=args["settings"], source_path="images/poster.jpg"
            ),
        }
        args["context"]["generated_content"] = {
            "article.rst": Article(
                "",
                settings=args["settings"],
                metadata={"slug": "article", "title": "Article"},
            )
        }

        # Article link will go to blog
        args["content"] = '<a href="{filename}article.rst">Article</a>'
        content = Page(**args).get_content("http://cool.site")
        self.assertEqual(
            content, '<a href="http://blog.cool.site/article.html">Article</a>'
        )

        # Page link will go to the main site
        args["content"] = '<a href="{index}">Index</a>'
        content = Page(**args).get_content("http://cool.site")
        self.assertEqual(content, '<a href="http://cool.site/index.html">Index</a>')

        # Image link will go to static
        args["content"] = '<img src="{static}/images/poster.jpg"/>'
        content = Page(**args).get_content("http://cool.site")
        self.assertEqual(
            content, '<img src="http://static.cool.site/images/poster.jpg"/>'
        )

        # Image link will go to static
        args["content"] = '<meta content="{static}/images/poster.jpg"/>'
        content = Page(**args).get_content("http://cool.site")
        self.assertEqual(
            content, '<meta content="http://static.cool.site/images/poster.jpg"/>'
        )

    def test_intrasite_link_escape(self):
        article = type("_DummyArticle", (object,), {"url": "article-spaces.html"})
        asset = type("_DummyAsset", (object,), {"url": "name@example.com"})

        args = self.page_kwargs.copy()
        args["settings"] = get_settings()
        args["source_path"] = "content"
        args["context"]["generated_content"] = {"article spaces.rst": article}
        args["context"]["static_content"] = {"name@example.com": asset}

        expected_output = (
            "A simple test with a "
            '<a href="http://notmyidea.org/article-spaces.html#anchor">link</a> '
            '<a href="http://notmyidea.org/name@example.com#anchor">file</a>'
        )

        # not escaped
        args["content"] = (
            "A simple test with a "
            '<a href="{filename}article spaces.rst#anchor">link</a> '
            '<a href="{static}name@example.com#anchor">file</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(content, expected_output)

        # html escaped
        args["content"] = (
            "A simple test with a "
            '<a href="{filename}article spaces.rst#anchor">link</a> '
            '<a href="{static}name&#64;example.com#anchor">file</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(content, expected_output)

        # url escaped
        args["content"] = (
            "A simple test with a "
            '<a href="{filename}article%20spaces.rst#anchor">link</a> '
            '<a href="{static}name%40example.com#anchor">file</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(content, expected_output)

        # html and url escaped
        args["content"] = (
            "A simple test with a "
            '<a href="{filename}article%20spaces.rst#anchor">link</a> '
            '<a href="{static}name&#64;example.com#anchor">file</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(content, expected_output)

    def test_intrasite_link_markdown_spaces(self):
        cls_name = "_DummyArticle"
        article = type(cls_name, (object,), {"url": "article-spaces.html"})

        args = self.page_kwargs.copy()
        args["settings"] = get_settings()
        args["source_path"] = "content"
        args["context"]["generated_content"] = {"article spaces.rst": article}

        # An intrasite link via filename with %20 as a space
        args["content"] = (
            'A simple test, with a <a href="|filename|article%20spaces.rst">link</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "A simple test, with a "
            '<a href="http://notmyidea.org/article-spaces.html">link</a>',
        )

    def test_intrasite_link_source_and_generated(self):
        """Test linking both to the source and the generated article"""
        cls_name = "_DummyAsset"

        args = self.page_kwargs.copy()
        args["settings"] = get_settings()
        args["source_path"] = "content"
        args["context"]["generated_content"] = {
            "article.rst": type(cls_name, (object,), {"url": "article.html"})
        }
        args["context"]["static_content"] = {
            "article.rst": type(cls_name, (object,), {"url": "article.rst"})
        }

        args["content"] = (
            "A simple test, with a link to an"
            '<a href="{filename}article.rst">article</a> and its'
            '<a href="{static}article.rst">source</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "A simple test, with a link to an"
            '<a href="http://notmyidea.org/article.html">article</a> and its'
            '<a href="http://notmyidea.org/article.rst">source</a>',
        )

    def test_intrasite_link_to_static_content_with_filename(self):
        """Test linking to a static resource with deprecated {filename}"""
        cls_name = "_DummyAsset"

        args = self.page_kwargs.copy()
        args["settings"] = get_settings()
        args["source_path"] = "content"
        args["context"]["static_content"] = {
            "poster.jpg": type(cls_name, (object,), {"url": "images/poster.jpg"})
        }

        args["content"] = (
            "A simple test, with a link to a"
            '<a href="{filename}poster.jpg">poster</a>'
        )
        content = Page(**args).get_content("http://notmyidea.org")
        self.assertEqual(
            content,
            "A simple test, with a link to a"
            '<a href="http://notmyidea.org/images/poster.jpg">poster</a>',
        )

    def test_multiple_authors(self):
        """Test article with multiple authors."""
        args = self.page_kwargs.copy()
        content = Page(**args)
        assert content.authors == [content.author]
        args["metadata"].pop("author")
        args["metadata"]["authors"] = [
            Author("First Author", DEFAULT_CONFIG),
            Author("Second Author", DEFAULT_CONFIG),
        ]
        content = Page(**args)
        assert content.authors
        assert content.author == content.authors[0]


class TestArticle(TestBase):
    def test_template(self):
        # Articles default to article, metadata overwrites
        default_article = Article(**self.page_kwargs)
        self.assertEqual("article", default_article.template)
        article_kwargs = self._copy_page_kwargs()
        article_kwargs["metadata"]["template"] = "custom"
        custom_article = Article(**article_kwargs)
        self.assertEqual("custom", custom_article.template)

    def test_slugify_category_author(self):
        settings = get_settings()
        settings["SLUG_REGEX_SUBSTITUTIONS"] = [
            (r"C#", "csharp"),
            (r"[^\w\s-]", ""),
            (r"(?u)\A\s*", ""),
            (r"(?u)\s*\Z", ""),
            (r"[-\s]+", "-"),
        ]
        settings["ARTICLE_URL"] = "{author}/{category}/{slug}/"
        settings["ARTICLE_SAVE_AS"] = "{author}/{category}/{slug}/index.html"
        article_kwargs = self._copy_page_kwargs()
        article_kwargs["metadata"]["author"] = Author("O'Brien", settings)
        article_kwargs["metadata"]["category"] = Category("C# & stuff", settings)
        article_kwargs["metadata"]["title"] = "fnord"
        article_kwargs["settings"] = settings
        article = Article(**article_kwargs)
        self.assertEqual(article.url, "obrien/csharp-stuff/fnord/")
        self.assertEqual(article.save_as, "obrien/csharp-stuff/fnord/index.html")

    def test_slugify_with_author_substitutions(self):
        settings = get_settings()
        settings["AUTHOR_REGEX_SUBSTITUTIONS"] = [
            ("Alexander Todorov", "atodorov"),
            ("Krasimir Tsonev", "krasimir"),
            (r"[^\w\s-]", ""),
            (r"(?u)\A\s*", ""),
            (r"(?u)\s*\Z", ""),
            (r"[-\s]+", "-"),
        ]
        settings["ARTICLE_URL"] = "blog/{author}/{slug}/"
        settings["ARTICLE_SAVE_AS"] = "blog/{author}/{slug}/index.html"
        article_kwargs = self._copy_page_kwargs()
        article_kwargs["metadata"]["author"] = Author("Alexander Todorov", settings)
        article_kwargs["metadata"]["title"] = "fnord"
        article_kwargs["settings"] = settings
        article = Article(**article_kwargs)
        self.assertEqual(article.url, "blog/atodorov/fnord/")
        self.assertEqual(article.save_as, "blog/atodorov/fnord/index.html")

    def test_slugify_category_with_dots(self):
        settings = get_settings()
        settings["CATEGORY_REGEX_SUBSTITUTIONS"] = [
            ("Fedora QA", "fedora.qa"),
        ]
        settings["ARTICLE_URL"] = "{category}/{slug}/"
        article_kwargs = self._copy_page_kwargs()
        article_kwargs["metadata"]["category"] = Category("Fedora QA", settings)
        article_kwargs["metadata"]["title"] = "This Week in Fedora QA"
        article_kwargs["settings"] = settings
        article = Article(**article_kwargs)
        self.assertEqual(article.url, "fedora.qa/this-week-in-fedora-qa/")

    def test_valid_save_as_detects_breakout(self):
        settings = get_settings()
        article_kwargs = self._copy_page_kwargs()
        article_kwargs["metadata"]["slug"] = "../foo"
        article_kwargs["settings"] = settings
        article = Article(**article_kwargs)
        self.assertFalse(article._has_valid_save_as())

    def test_valid_save_as_detects_breakout_to_root(self):
        settings = get_settings()
        article_kwargs = self._copy_page_kwargs()
        article_kwargs["metadata"]["slug"] = "/foo"
        article_kwargs["settings"] = settings
        article = Article(**article_kwargs)
        self.assertFalse(article._has_valid_save_as())

    def test_valid_save_as_passes_valid(self):
        settings = get_settings()
        article_kwargs = self._copy_page_kwargs()
        article_kwargs["metadata"]["slug"] = "foo"
        article_kwargs["settings"] = settings
        article = Article(**article_kwargs)
        self.assertTrue(article._has_valid_save_as())


class TestStatic(LoggedTestCase):
    def setUp(self):
        super().setUp()
        self.settings = get_settings(
            STATIC_SAVE_AS="{path}",
            STATIC_URL="{path}",
            PAGE_SAVE_AS=os.path.join("outpages", "{slug}.html"),
            PAGE_URL="outpages/{slug}.html",
        )
        self.context = get_context(self.settings)

        self.static = Static(
            content=None,
            metadata={},
            settings=self.settings,
            source_path=posix_join("dir", "foo.jpg"),
            context=self.context,
        )

        self.context["static_content"][self.static.source_path] = self.static

    def tearDown(self):
        pass

    def test_attach_to_same_dir(self):
        """attach_to() overrides a static file's save_as and url."""
        page = Page(
            content="fake page",
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "fakepage.md"),
        )
        self.static.attach_to(page)

        expected_save_as = os.path.join("outpages", "foo.jpg")
        self.assertEqual(self.static.save_as, expected_save_as)
        self.assertEqual(self.static.url, path_to_url(expected_save_as))

    def test_attach_to_parent_dir(self):
        """attach_to() preserves dirs inside the linking document dir."""
        page = Page(
            content="fake page",
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path="fakepage.md",
        )
        self.static.attach_to(page)

        expected_save_as = os.path.join("outpages", "dir", "foo.jpg")
        self.assertEqual(self.static.save_as, expected_save_as)
        self.assertEqual(self.static.url, path_to_url(expected_save_as))

    def test_attach_to_other_dir(self):
        """attach_to() ignores dirs outside the linking document dir."""
        page = Page(
            content="fake page",
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
        )
        self.static.attach_to(page)

        expected_save_as = os.path.join("outpages", "foo.jpg")
        self.assertEqual(self.static.save_as, expected_save_as)
        self.assertEqual(self.static.url, path_to_url(expected_save_as))

    def test_attach_to_ignores_subsequent_calls(self):
        """attach_to() does nothing when called a second time."""
        page = Page(
            content="fake page",
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "fakepage.md"),
        )

        self.static.attach_to(page)

        otherdir_settings = self.settings.copy()
        otherdir_settings.update(
            {
                "PAGE_SAVE_AS": os.path.join("otherpages", "{slug}.html"),
                "PAGE_URL": "otherpages/{slug}.html",
            }
        )
        otherdir_page = Page(
            content="other page",
            metadata={"title": "otherpage"},
            settings=otherdir_settings,
            source_path=os.path.join("dir", "otherpage.md"),
        )

        self.static.attach_to(otherdir_page)

        otherdir_save_as = os.path.join("otherpages", "foo.jpg")
        self.assertNotEqual(self.static.save_as, otherdir_save_as)
        self.assertNotEqual(self.static.url, path_to_url(otherdir_save_as))

    def test_attach_to_does_nothing_after_save_as_referenced(self):
        """attach_to() does nothing if the save_as was already referenced.
        (For example, by a {static} link an a document processed earlier.)
        """
        original_save_as = self.static.save_as

        page = Page(
            content="fake page",
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "fakepage.md"),
        )
        self.static.attach_to(page)

        self.assertEqual(self.static.save_as, original_save_as)
        self.assertEqual(self.static.url, path_to_url(original_save_as))

    def test_attach_to_does_nothing_after_url_referenced(self):
        """attach_to() does nothing if the url was already referenced.
        (For example, by a {static} link an a document processed earlier.)
        """
        original_url = self.static.url

        page = Page(
            content="fake page",
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "fakepage.md"),
        )
        self.static.attach_to(page)

        self.assertEqual(self.static.save_as, self.static.source_path)
        self.assertEqual(self.static.url, original_url)

    def test_attach_to_does_not_override_an_override(self):
        """attach_to() does not override paths that were overridden elsewhere.
        (For example, by the user with EXTRA_PATH_METADATA)
        """
        customstatic = Static(
            content=None,
            metadata={"save_as": "customfoo.jpg", "url": "customfoo.jpg"},
            settings=self.settings,
            source_path=os.path.join("dir", "foo.jpg"),
            context=self.settings.copy(),
        )

        page = Page(
            content="fake page",
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "fakepage.md"),
        )

        customstatic.attach_to(page)

        self.assertEqual(customstatic.save_as, "customfoo.jpg")
        self.assertEqual(customstatic.url, "customfoo.jpg")

    def test_attach_link_syntax(self):
        """{attach} link syntax triggers output path override & url replacement."""
        html = '<a href="{attach}../foo.jpg">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertNotEqual(
            content, html, "{attach} link syntax did not trigger URL replacement."
        )

        expected_save_as = os.path.join("outpages", "foo.jpg")
        self.assertEqual(self.static.save_as, expected_save_as)
        self.assertEqual(self.static.url, path_to_url(expected_save_as))

    def test_tag_link_syntax(self):
        "{tag} link syntax triggers url replacement."

        html = '<a href="{tag}foo">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertNotEqual(content, html)

    def test_category_link_syntax(self):
        "{category} link syntax triggers url replacement."

        html = '<a href="{category}foo">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertNotEqual(content, html)

    def test_author_link_syntax(self):
        "{author} link syntax triggers url replacement."

        html = '<a href="{author}foo">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertNotEqual(content, html)

    def test_index_link_syntax(self):
        "{index} link syntax triggers url replacement."

        html = '<a href="{index}">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertNotEqual(content, html)

        expected_html = (
            '<a href="'
            + "/".join((self.settings["SITEURL"], self.settings["INDEX_SAVE_AS"]))
            + '">link</a>'
        )
        self.assertEqual(content, expected_html)

    def test_unknown_link_syntax(self):
        "{unknown} link syntax should trigger warning."

        html = '<a href="{unknown}foo">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertEqual(content, html)
        self.assertLogCountEqual(
            count=1,
            msg="Replacement Indicator 'unknown' not recognized in '{unknown}foo', "
            "skipping replacement",
            level=logging.WARNING,
        )

    def test_link_to_unknown_file(self):
        "{filename} link to unknown file should trigger warning."

        html = '<a href="{filename}foo">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertEqual(content, html)
        self.assertLogCountEqual(
            count=1,
            msg="Unable to find 'foo', skipping url replacement.",
            level=logging.WARNING,
        )

    def test_index_link_syntax_with_spaces(self):
        """{index} link syntax triggers url replacement
        with spaces around the equal sign."""

        html = '<a href = "{index}">link</a>'
        page = Page(
            content=html,
            metadata={"title": "fakepage"},
            settings=self.settings,
            source_path=os.path.join("dir", "otherdir", "fakepage.md"),
            context=self.context,
        )
        content = page.get_content("")

        self.assertNotEqual(content, html)

        expected_html = (
            '<a href = "'
            + "/".join((self.settings["SITEURL"], self.settings["INDEX_SAVE_AS"]))
            + '">link</a>'
        )
        self.assertEqual(content, expected_html)

    def test_not_save_as_draft(self):
        """Static.save_as is not affected by draft status."""

        static = Static(
            content=None,
            metadata={
                "status": "draft",
            },
            settings=self.settings,
            source_path=os.path.join("dir", "foo.jpg"),
            context=self.settings.copy(),
        )

        expected_save_as = posixize_path(os.path.join("dir", "foo.jpg"))
        self.assertEqual(static.status, "draft")
        self.assertEqual(static.save_as, expected_save_as)
        self.assertEqual(static.url, path_to_url(expected_save_as))



import locale

from jinja2.utils import generate_lorem_ipsum

from pelican.contents import Article, Author
from pelican.paginator import Paginator
from pelican.settings import DEFAULT_CONFIG
from pelican.tests.support import get_settings, unittest

# generate one paragraph, enclosed with <p>
TEST_CONTENT = str(generate_lorem_ipsum(n=1))
TEST_SUMMARY = generate_lorem_ipsum(n=1, html=False)


class TestPage(unittest.TestCase):
    def setUp(self):
        super().setUp()
        self.old_locale = locale.setlocale(locale.LC_ALL)
        locale.setlocale(locale.LC_ALL, "C")
        self.page_kwargs = {
            "content": TEST_CONTENT,
            "context": {
                "localsiteurl": "",
            },
            "metadata": {
                "summary": TEST_SUMMARY,
                "title": "foo bar",
            },
            "source_path": "/path/to/file/foo.ext",
        }

    def tearDown(self):
        locale.setlocale(locale.LC_ALL, self.old_locale)

    def test_save_as_preservation(self):
        settings = get_settings()
        # fix up pagination rules
        from pelican.paginator import PaginationRule

        pagination_rules = [
            PaginationRule(*r)
            for r in settings.get(
                "PAGINATION_PATTERNS",
                DEFAULT_CONFIG["PAGINATION_PATTERNS"],
            )
        ]
        settings["PAGINATION_PATTERNS"] = sorted(
            pagination_rules,
            key=lambda r: r[0],
        )

        self.page_kwargs["metadata"]["author"] = Author("Blogger", settings)
        object_list = [Article(**self.page_kwargs), Article(**self.page_kwargs)]
        paginator = Paginator("foobar.foo", "foobar/foo", object_list, settings)
        page = paginator.page(1)
        self.assertEqual(page.save_as, "foobar.foo")

    def test_custom_pagination_pattern(self):
        from pelican.paginator import PaginationRule

        settings = get_settings()
        settings["PAGINATION_PATTERNS"] = [
            PaginationRule(*r)
            for r in [
                (1, "/{url}", "{base_name}/index.html"),
                (2, "/{url}{number}/", "{base_name}/{number}/index.html"),
            ]
        ]

        self.page_kwargs["metadata"]["author"] = Author("Blogger", settings)
        object_list = [Article(**self.page_kwargs), Article(**self.page_kwargs)]
        paginator = Paginator(
            "blog/index.html", "//blog.my.site/", object_list, settings, 1
        )
        # The URL *has to* stay absolute (with // in the front), so verify that
        page1 = paginator.page(1)
        self.assertEqual(page1.save_as, "blog/index.html")
        self.assertEqual(page1.url, "//blog.my.site/")
        page2 = paginator.page(2)
        self.assertEqual(page2.save_as, "blog/2/index.html")
        self.assertEqual(page2.url, "//blog.my.site/2/")

    def test_custom_pagination_pattern_last_page(self):
        from pelican.paginator import PaginationRule

        settings = get_settings()
        settings["PAGINATION_PATTERNS"] = [
            PaginationRule(*r)
            for r in [
                (1, "/{url}1/", "{base_name}/1/index.html"),
                (2, "/{url}{number}/", "{base_name}/{number}/index.html"),
                (-1, "/{url}", "{base_name}/index.html"),
            ]
        ]

        self.page_kwargs["metadata"]["author"] = Author("Blogger", settings)
        object_list = [
            Article(**self.page_kwargs),
            Article(**self.page_kwargs),
            Article(**self.page_kwargs),
        ]
        paginator = Paginator(
            "blog/index.html", "//blog.my.site/", object_list, settings, 1
        )
        # The URL *has to* stay absolute (with // in the front), so verify that
        page1 = paginator.page(1)
        self.assertEqual(page1.save_as, "blog/1/index.html")
        self.assertEqual(page1.url, "//blog.my.site/1/")
        page2 = paginator.page(2)
        self.assertEqual(page2.save_as, "blog/2/index.html")
        self.assertEqual(page2.url, "//blog.my.site/2/")
        page3 = paginator.page(3)
        self.assertEqual(page3.save_as, "blog/index.html")
        self.assertEqual(page3.url, "//blog.my.site/")



from pelican.tests.support import unittest
from pelican.urlwrappers import Author, Category, Tag, URLWrapper


class TestURLWrapper(unittest.TestCase):
    def test_ordering(self):
        # URLWrappers are sorted by name
        wrapper_a = URLWrapper(name="first", settings={})
        wrapper_b = URLWrapper(name="last", settings={})
        self.assertFalse(wrapper_a > wrapper_b)
        self.assertFalse(wrapper_a >= wrapper_b)
        self.assertFalse(wrapper_a == wrapper_b)
        self.assertTrue(wrapper_a != wrapper_b)
        self.assertTrue(wrapper_a <= wrapper_b)
        self.assertTrue(wrapper_a < wrapper_b)
        wrapper_b.name = "first"
        self.assertFalse(wrapper_a > wrapper_b)
        self.assertTrue(wrapper_a >= wrapper_b)
        self.assertTrue(wrapper_a == wrapper_b)
        self.assertFalse(wrapper_a != wrapper_b)
        self.assertTrue(wrapper_a <= wrapper_b)
        self.assertFalse(wrapper_a < wrapper_b)
        wrapper_a.name = "last"
        self.assertTrue(wrapper_a > wrapper_b)
        self.assertTrue(wrapper_a >= wrapper_b)
        self.assertFalse(wrapper_a == wrapper_b)
        self.assertTrue(wrapper_a != wrapper_b)
        self.assertFalse(wrapper_a <= wrapper_b)
        self.assertFalse(wrapper_a < wrapper_b)

    def test_equality(self):
        tag = Tag("test", settings={})
        cat = Category("test", settings={})
        author = Author("test", settings={})

        # same name, but different class
        self.assertNotEqual(tag, cat)
        self.assertNotEqual(tag, author)

        # should be equal vs text representing the same name
        self.assertEqual(tag, "test")

        # should not be equal vs binary
        self.assertNotEqual(tag, b"test")

        # Tags describing the same should be equal
        tag_equal = Tag("Test", settings={})
        self.assertEqual(tag, tag_equal)

        # Author describing the same should be equal
        author_equal = Author("Test", settings={})
        self.assertEqual(author, author_equal)

        cat_ascii = Category("指導書", settings={})
        self.assertEqual(cat_ascii, "zhi dao shu")

    def test_slugify_with_substitutions_and_dots(self):
        tag = Tag(
            "Tag Dot",
            settings={
                "TAG_REGEX_SUBSTITUTIONS": [
                    ("Tag Dot", "tag.dot"),
                ]
            },
        )
        cat = Category(
            "Category Dot",
            settings={
                "CATEGORY_REGEX_SUBSTITUTIONS": [
                    ("Category Dot", "cat.dot"),
                ]
            },
        )

        self.assertEqual(tag.slug, "tag.dot")
        self.assertEqual(cat.slug, "cat.dot")

    def test_author_slug_substitutions(self):
        settings = {
            "AUTHOR_REGEX_SUBSTITUTIONS": [
                ("Alexander Todorov", "atodorov"),
                ("Krasimir Tsonev", "krasimir"),
                (r"[^\w\s-]", ""),
                (r"(?u)\A\s*", ""),
                (r"(?u)\s*\Z", ""),
                (r"[-\s]+", "-"),
            ]
        }

        author1 = Author("Mr. Senko", settings=settings)
        author2 = Author("Alexander Todorov", settings=settings)
        author3 = Author("Krasimir Tsonev", settings=settings)

        self.assertEqual(author1.slug, "mr-senko")
        self.assertEqual(author2.slug, "atodorov")
        self.assertEqual(author3.slug, "krasimir")



import os
import re
from posixpath import join as posix_join
from unittest.mock import patch

from pelican.settings import DEFAULT_CONFIG
from pelican.tests.support import (
    TestCaseWithCLocale,
    mute,
    skipIfNoExecutable,
    temporary_folder,
    unittest,
)
from pelican.tools.pelican_import import (
    blogger2fields,
    build_header,
    build_markdown_header,
    decode_wp_content,
    download_attachments,
    fields2pelican,
    get_attachments,
    medium_slug,
    mediumpost2fields,
    mediumposts2fields,
    strip_medium_post_content,
    tumblr2fields,
    wp2fields,
)
from pelican.utils import path_to_file_url, slugify

CUR_DIR = os.path.abspath(os.path.dirname(__file__))
BLOGGER_XML_SAMPLE = os.path.join(CUR_DIR, "content", "bloggerexport.xml")
WORDPRESS_XML_SAMPLE = os.path.join(CUR_DIR, "content", "wordpressexport.xml")
WORDPRESS_ENCODED_CONTENT_SAMPLE = os.path.join(
    CUR_DIR, "content", "wordpress_content_encoded"
)
WORDPRESS_DECODED_CONTENT_SAMPLE = os.path.join(
    CUR_DIR, "content", "wordpress_content_decoded"
)

try:
    from bs4 import BeautifulSoup
except ImportError:
    BeautifulSoup = False

try:
    import bs4.builder._lxml as LXML
except ImportError:
    LXML = False


@skipIfNoExecutable(["pandoc", "--version"])
@unittest.skipUnless(BeautifulSoup, "Needs BeautifulSoup module")
class TestBloggerXmlImporter(TestCaseWithCLocale):
    def setUp(self):
        super().setUp()
        self.posts = blogger2fields(BLOGGER_XML_SAMPLE)

    def test_recognise_kind_and_title(self):
        """Check that importer only outputs pages, articles and comments,
        that these are correctly identified and that titles are correct.
        """
        test_posts = list(self.posts)
        kinds = {x[8] for x in test_posts}
        self.assertEqual({"page", "article", "comment"}, kinds)
        page_titles = {x[0] for x in test_posts if x[8] == "page"}
        self.assertEqual({"Test page", "Test page 2"}, page_titles)
        article_titles = {x[0] for x in test_posts if x[8] == "article"}
        self.assertEqual(
            {"Black as Egypt's Night", "The Steel Windpipe"}, article_titles
        )
        comment_titles = {x[0] for x in test_posts if x[8] == "comment"}
        self.assertEqual(
            {"Mishka, always a pleasure to read your adventures!..."}, comment_titles
        )

    def test_recognise_status_with_correct_filename(self):
        """Check that importerer outputs only statuses 'published' and 'draft',
        that these are correctly identified and that filenames are correct.
        """
        test_posts = list(self.posts)
        statuses = {x[7] for x in test_posts}
        self.assertEqual({"published", "draft"}, statuses)

        draft_filenames = {x[2] for x in test_posts if x[7] == "draft"}
        # draft filenames are id-based
        self.assertEqual(
            {"page-4386962582497458967", "post-1276418104709695660"}, draft_filenames
        )

        published_filenames = {x[2] for x in test_posts if x[7] == "published"}
        # published filenames are url-based, except comments
        self.assertEqual(
            {"the-steel-windpipe", "test-page", "post-5590533389087749201"},
            published_filenames,
        )


@skipIfNoExecutable(["pandoc", "--version"])
@unittest.skipUnless(BeautifulSoup, "Needs BeautifulSoup module")
class TestWordpressXmlImporter(TestCaseWithCLocale):
    def setUp(self):
        super().setUp()
        self.posts = wp2fields(WORDPRESS_XML_SAMPLE)
        self.custposts = wp2fields(WORDPRESS_XML_SAMPLE, True)

    def test_ignore_empty_posts(self):
        self.assertTrue(self.posts)
        for (
            title,
            _content,
            _fname,
            _date,
            _author,
            _categ,
            _tags,
            _status,
            _kind,
            _format,
        ) in self.posts:
            self.assertTrue(title.strip())

    def test_recognise_page_kind(self):
        """Check that we recognise pages in wordpress, as opposed to posts"""
        self.assertTrue(self.posts)
        # Collect (title, filename, kind) of non-empty posts recognised as page
        pages_data = []
        for (
            title,
            _content,
            fname,
            _date,
            _author,
            _categ,
            _tags,
            _status,
            kind,
            _format,
        ) in self.posts:
            if kind == "page":
                pages_data.append((title, fname))
        self.assertEqual(2, len(pages_data))
        self.assertEqual(("Page", "contact"), pages_data[0])
        self.assertEqual(("Empty Page", "empty"), pages_data[1])

    def test_dirpage_directive_for_page_kind(self):
        silent_f2p = mute(True)(fields2pelican)
        test_post = filter(lambda p: p[0].startswith("Empty Page"), self.posts)
        with temporary_folder() as temp:
            fname = next(iter(silent_f2p(test_post, "markdown", temp, dirpage=True)))
            self.assertTrue(fname.endswith(f"pages{os.path.sep}empty.md"))

    def test_dircat(self):
        silent_f2p = mute(True)(fields2pelican)
        test_posts = [
            post
            for post in self.posts
            # check post has a category
            if len(post[5]) > 0
        ]
        with temporary_folder() as temp:
            fnames = list(silent_f2p(test_posts, "markdown", temp, dircat=True))
        subs = DEFAULT_CONFIG["SLUG_REGEX_SUBSTITUTIONS"]
        index = 0
        for post in test_posts:
            name = post[2]
            category = slugify(post[5][0], regex_subs=subs, preserve_case=True)
            name += ".md"
            filename = os.path.join(category, name)
            out_name = fnames[index]
            self.assertTrue(out_name.endswith(filename))
            index += 1

    def test_unless_custom_post_all_items_should_be_pages_or_posts(self):
        self.assertTrue(self.posts)
        pages_data = []
        for (
            title,
            _content,
            fname,
            _date,
            _author,
            _categ,
            _tags,
            _status,
            kind,
            _format,
        ) in self.posts:
            if kind in {"page", "article"}:
                pass
            else:
                pages_data.append((title, fname))
        self.assertEqual(0, len(pages_data))

    def test_recognise_custom_post_type(self):
        self.assertTrue(self.custposts)
        cust_data = []
        for (
            title,
            _content,
            _fname,
            _date,
            _author,
            _categ,
            _tags,
            _status,
            kind,
            _format,
        ) in self.custposts:
            if kind in {"page", "article"}:
                pass
            else:
                cust_data.append((title, kind))
        self.assertEqual(3, len(cust_data))
        self.assertEqual(("A custom post in category 4", "custom1"), cust_data[0])
        self.assertEqual(("A custom post in category 5", "custom1"), cust_data[1])
        self.assertEqual(
            ("A 2nd custom post type also in category 5", "custom2"), cust_data[2]
        )

    def test_custom_posts_put_in_own_dir(self):
        silent_f2p = mute(True)(fields2pelican)
        test_posts = []
        for post in self.custposts:
            # check post kind
            if post[8] == "article" or post[8] == "page":
                pass
            else:
                test_posts.append(post)
        with temporary_folder() as temp:
            fnames = list(silent_f2p(test_posts, "markdown", temp, wp_custpost=True))
        index = 0
        for post in test_posts:
            name = post[2]
            kind = post[8]
            name += ".md"
            filename = os.path.join(kind, name)
            out_name = fnames[index]
            self.assertTrue(out_name.endswith(filename))
            index += 1

    def test_custom_posts_put_in_own_dir_and_catagory_sub_dir(self):
        silent_f2p = mute(True)(fields2pelican)
        test_posts = []
        for post in self.custposts:
            # check post kind
            if post[8] == "article" or post[8] == "page":
                pass
            else:
                test_posts.append(post)
        with temporary_folder() as temp:
            fnames = list(
                silent_f2p(test_posts, "markdown", temp, wp_custpost=True, dircat=True)
            )
        subs = DEFAULT_CONFIG["SLUG_REGEX_SUBSTITUTIONS"]
        index = 0
        for post in test_posts:
            name = post[2]
            kind = post[8]
            category = slugify(post[5][0], regex_subs=subs, preserve_case=True)
            name += ".md"
            filename = os.path.join(kind, category, name)
            out_name = fnames[index]
            self.assertTrue(out_name.endswith(filename))
            index += 1

    def test_wp_custpost_true_dirpage_false(self):
        # pages should only be put in their own directory when dirpage = True
        silent_f2p = mute(True)(fields2pelican)
        test_posts = [
            post
            for post in self.custposts
            # check post kind
            if post[8] == "page"
        ]
        with temporary_folder() as temp:
            fnames = list(
                silent_f2p(
                    test_posts, "markdown", temp, wp_custpost=True, dirpage=False
                )
            )
        index = 0
        for post in test_posts:
            name = post[2]
            name += ".md"
            filename = os.path.join("pages", name)
            out_name = fnames[index]
            self.assertFalse(out_name.endswith(filename))

    def test_can_toggle_raw_html_code_parsing(self):
        test_posts = list(self.posts)

        def r(f):
            with open(f, encoding="utf-8") as infile:
                return infile.read()

        silent_f2p = mute(True)(fields2pelican)

        with temporary_folder() as temp:
            rst_files = (r(f) for f in silent_f2p(test_posts, "markdown", temp))
            self.assertTrue(any("<iframe" in rst for rst in rst_files))
            rst_files = (
                r(f) for f in silent_f2p(test_posts, "markdown", temp, strip_raw=True)
            )
            self.assertFalse(any("<iframe" in rst for rst in rst_files))
            # no effect in rst
            rst_files = (r(f) for f in silent_f2p(test_posts, "rst", temp))
            self.assertFalse(any("<iframe" in rst for rst in rst_files))
            rst_files = (
                r(f) for f in silent_f2p(test_posts, "rst", temp, strip_raw=True)
            )
            self.assertFalse(any("<iframe" in rst for rst in rst_files))

    def test_decode_html_entities_in_titles(self):
        test_posts = [post for post in self.posts if post[2] == "html-entity-test"]
        self.assertEqual(len(test_posts), 1)

        post = test_posts[0]
        title = post[0]
        self.assertTrue(
            title,
            "A normal post with some <html> entities in "
            "the title. You can't miss them.",
        )
        self.assertNotIn("&", title)

    def test_decode_wp_content_returns_empty(self):
        """Check that given an empty string we return an empty string."""
        self.assertEqual(decode_wp_content(""), "")

    def test_decode_wp_content(self):
        """Check that we can decode a wordpress content string."""
        with open(WORDPRESS_ENCODED_CONTENT_SAMPLE) as encoded_file:
            encoded_content = encoded_file.read()
            with open(WORDPRESS_DECODED_CONTENT_SAMPLE) as decoded_file:
                decoded_content = decoded_file.read()
                self.assertEqual(
                    decode_wp_content(encoded_content, br=False), decoded_content
                )

    def test_preserve_verbatim_formatting(self):
        def r(f):
            with open(f, encoding="utf-8") as infile:
                return infile.read()

        silent_f2p = mute(True)(fields2pelican)
        test_post = filter(lambda p: p[0].startswith("Code in List"), self.posts)
        with temporary_folder() as temp:
            md = next(r(f) for f in silent_f2p(test_post, "markdown", temp))
            self.assertTrue(re.search(r"\s+a = \[1, 2, 3\]", md))
            self.assertTrue(re.search(r"\s+b = \[4, 5, 6\]", md))

            for_line = re.search(r"\s+for i in zip\(a, b\):", md).group(0)
            print_line = re.search(r"\s+print i", md).group(0)
            self.assertTrue(for_line.rindex("for") < print_line.rindex("print"))

    def test_code_in_list(self):
        def r(f):
            with open(f, encoding="utf-8") as infile:
                return infile.read()

        silent_f2p = mute(True)(fields2pelican)
        test_post = filter(lambda p: p[0].startswith("Code in List"), self.posts)
        with temporary_folder() as temp:
            md = next(r(f) for f in silent_f2p(test_post, "markdown", temp))
            sample_line = re.search(r"- This is a code sample", md).group(0)
            code_line = re.search(r"\s+a = \[1, 2, 3\]", md).group(0)
            self.assertTrue(sample_line.rindex("This") < code_line.rindex("a"))

    def test_dont_use_smart_quotes(self):
        def r(f):
            with open(f, encoding="utf-8") as infile:
                return infile.read()

        silent_f2p = mute(True)(fields2pelican)
        test_post = filter(lambda p: p[0].startswith("Post with raw data"), self.posts)
        with temporary_folder() as temp:
            md = next(r(f) for f in silent_f2p(test_post, "markdown", temp))
            escaped_quotes = re.search(r'\\[\'"“”‘’]', md)  # noqa: RUF001
            self.assertFalse(escaped_quotes)

    def test_convert_caption_to_figure(self):
        def r(f):
            with open(f, encoding="utf-8") as infile:
                return infile.read()

        silent_f2p = mute(True)(fields2pelican)
        test_post = filter(lambda p: p[0].startswith("Caption on image"), self.posts)
        with temporary_folder() as temp:
            md = next(r(f) for f in silent_f2p(test_post, "markdown", temp))

        caption = re.search(r"\[caption", md)
        self.assertFalse(caption)

        for occurence in [
            "/theme/img/xpelican.png.pagespeed.ic.Rjep0025-y.png",
            "/theme/img/xpelican-3.png.pagespeed.ic.m-NAIdRCOM.png",
            "/theme/img/xpelican.png.pagespeed.ic.Rjep0025-y.png",
            "This is a pelican",
            "This also a pelican",
            "Yet another pelican",
        ]:
            # pandoc 2.x converts into ![text](src)
            # pandoc 3.x converts into <figure>src<figcaption>text</figcaption></figure>
            self.assertIn(occurence, md)


class TestBuildHeader(unittest.TestCase):
    def test_build_header(self):
        header = build_header("test", None, None, None, None, None)
        self.assertEqual(header, "test\n####\n\n")

    def test_build_header_with_fields(self):
        header_data = [
            "Test Post",
            "2014-11-04",
            "Alexis Métaireau",
            ["Programming"],
            ["Pelican", "Python"],
            "test-post",
        ]

        expected_docutils = "\n".join(
            [
                "Test Post",
                "#########",
                ":date: 2014-11-04",
                ":author: Alexis Métaireau",
                ":category: Programming",
                ":tags: Pelican, Python",
                ":slug: test-post",
                "\n",
            ]
        )

        expected_md = "\n".join(
            [
                "Title: Test Post",
                "Date: 2014-11-04",
                "Author: Alexis Métaireau",
                "Category: Programming",
                "Tags: Pelican, Python",
                "Slug: test-post",
                "\n",
            ]
        )

        self.assertEqual(build_header(*header_data), expected_docutils)
        self.assertEqual(build_markdown_header(*header_data), expected_md)

    def test_build_header_with_east_asian_characters(self):
        header = build_header(
            "これは広い幅の文字だけで構成されたタイトルです",
            None,
            None,
            None,
            None,
            None,
        )

        self.assertEqual(
            header,
            (
                "これは広い幅の文字だけで構成されたタイトルです\n"
                "##############################################"
                "\n\n"
            ),
        )

    def test_galleries_added_to_header(self):
        header = build_header(
            "test",
            None,
            None,
            None,
            None,
            None,
            attachments=["output/test1", "output/test2"],
        )
        self.assertEqual(
            header, ("test\n####\n:attachments: output/test1, output/test2\n\n")
        )

    def test_galleries_added_to_markdown_header(self):
        header = build_markdown_header(
            "test",
            None,
            None,
            None,
            None,
            None,
            attachments=["output/test1", "output/test2"],
        )
        self.assertEqual(
            header, "Title: test\nAttachments: output/test1, output/test2\n\n"
        )


@unittest.skipUnless(BeautifulSoup, "Needs BeautifulSoup module")
@unittest.skipUnless(LXML, "Needs lxml module")
class TestWordpressXMLAttachements(TestCaseWithCLocale):
    def setUp(self):
        super().setUp()
        self.attachments = get_attachments(WORDPRESS_XML_SAMPLE)

    def test_recognise_attachments(self):
        self.assertTrue(self.attachments)
        self.assertEqual(3, len(self.attachments.keys()))

    def test_attachments_associated_with_correct_post(self):
        self.assertTrue(self.attachments)
        for post in self.attachments.keys():
            if post is None:
                expected = {
                    (
                        "https://upload.wikimedia.org/wikipedia/commons/"
                        "thumb/2/2c/Pelican_lakes_entrance02.jpg/"
                        "240px-Pelican_lakes_entrance02.jpg"
                    )
                }
                self.assertEqual(self.attachments[post], expected)
            elif post == "with-excerpt":
                expected_invalid = (
                    "http://thisurlisinvalid.notarealdomain/not_an_image.jpg"
                )
                expected_pelikan = (
                    "http://en.wikipedia.org/wiki/File:Pelikan_Walvis_Bay.jpg"
                )
                self.assertEqual(
                    self.attachments[post], {expected_invalid, expected_pelikan}
                )
            elif post == "with-tags":
                expected_invalid = "http://thisurlisinvalid.notarealdomain"
                self.assertEqual(self.attachments[post], {expected_invalid})
            else:
                self.fail(f"all attachments should match to a filename or None, {post}")

    def test_download_attachments(self):
        real_file = os.path.join(CUR_DIR, "content/article.rst")
        good_url = path_to_file_url(real_file)
        bad_url = "http://localhost:1/not_a_file.txt"
        silent_da = mute()(download_attachments)
        with temporary_folder() as temp:
            locations = list(silent_da(temp, [good_url, bad_url]))
            self.assertEqual(1, len(locations))
            directory = locations[0]
            self.assertTrue(
                directory.endswith(posix_join("content", "article.rst")), directory
            )


class TestTumblrImporter(TestCaseWithCLocale):
    @patch("pelican.tools.pelican_import._get_tumblr_posts")
    def test_posts(self, get):
        def get_posts(api_key, blogname, offset=0):
            if offset > 0:
                return []

            return [
                {
                    "type": "photo",
                    "blog_name": "testy",
                    "date": "2019-11-07 21:26:40 UTC",
                    "timestamp": 1573162000,
                    "format": "html",
                    "slug": "a-slug",
                    "tags": ["economics"],
                    "state": "published",
                    "photos": [
                        {
                            "caption": "",
                            "original_size": {
                                "url": "https://..fccdc2360ba7182a.jpg",
                                "width": 634,
                                "height": 789,
                            },
                        }
                    ],
                }
            ]

        get.side_effect = get_posts

        posts = list(tumblr2fields("api_key", "blogname"))
        self.assertEqual(
            [
                (
                    "Photo",
                    '<img alt="" src="https://..fccdc2360ba7182a.jpg" />\n',
                    "2019-11-07-a-slug",
                    "2019-11-07 21:26:40+0000",
                    "testy",
                    ["photo"],
                    ["economics"],
                    "published",
                    "article",
                    "html",
                )
            ],
            posts,
            posts,
        )

    @patch("pelican.tools.pelican_import._get_tumblr_posts")
    def test_video_embed(self, get):
        def get_posts(api_key, blogname, offset=0):
            if offset > 0:
                return []

            return [
                {
                    "type": "video",
                    "blog_name": "testy",
                    "slug": "the-slug",
                    "date": "2017-07-07 20:31:41 UTC",
                    "timestamp": 1499459501,
                    "state": "published",
                    "format": "html",
                    "tags": [],
                    "source_url": "https://href.li/?https://www.youtube.com/a",
                    "source_title": "youtube.com",
                    "caption": "<p>Caption</p>",
                    "player": [
                        {"width": 250, "embed_code": "<iframe>1</iframe>"},
                        {"width": 400, "embed_code": "<iframe>2</iframe>"},
                        {"width": 500, "embed_code": "<iframe>3</iframe>"},
                    ],
                    "video_type": "youtube",
                }
            ]

        get.side_effect = get_posts

        posts = list(tumblr2fields("api_key", "blogname"))
        self.assertEqual(
            [
                (
                    "youtube.com",
                    '<p><a href="https://href.li/?'
                    'https://www.youtube.com/a">via</a></p>\n<p>Caption</p>'
                    "<iframe>1</iframe>\n"
                    "<iframe>2</iframe>\n"
                    "<iframe>3</iframe>\n",
                    "2017-07-07-the-slug",
                    "2017-07-07 20:31:41+0000",
                    "testy",
                    ["video"],
                    [],
                    "published",
                    "article",
                    "html",
                )
            ],
            posts,
            posts,
        )

    @patch("pelican.tools.pelican_import._get_tumblr_posts")
    def test_broken_video_embed(self, get):
        def get_posts(api_key, blogname, offset=0):
            if offset > 0:
                return []

            return [
                {
                    "type": "video",
                    "blog_name": "testy",
                    "slug": "the-slug",
                    "date": "2016-08-14 16:37:35 UTC",
                    "timestamp": 1471192655,
                    "state": "published",
                    "format": "html",
                    "tags": ["interviews"],
                    "source_url": "https://href.li/?https://www.youtube.com/watch?v=b",
                    "source_title": "youtube.com",
                    "caption": "<p>Caption</p>",
                    "player": [
                        {
                            "width": 250,
                            # If video is gone, embed_code is False
                            "embed_code": False,
                        },
                        {"width": 400, "embed_code": False},
                        {"width": 500, "embed_code": False},
                    ],
                    "video_type": "youtube",
                }
            ]

        get.side_effect = get_posts

        posts = list(tumblr2fields("api_key", "blogname"))
        self.assertEqual(
            [
                (
                    "youtube.com",
                    '<p><a href="https://href.li/?https://www.youtube.com/watch?'
                    'v=b">via</a></p>\n<p>Caption</p>'
                    "<p>(This video isn't available anymore.)</p>\n",
                    "2016-08-14-the-slug",
                    "2016-08-14 16:37:35+0000",
                    "testy",
                    ["video"],
                    ["interviews"],
                    "published",
                    "article",
                    "html",
                )
            ],
            posts,
            posts,
        )


class TestMediumImporter(TestCaseWithCLocale):
    def setUp(self):
        super().setUp()
        self.test_content_root = "pelican/tests/content"
        # The content coming out of parsing is similar, but not the same.
        # Beautiful soup rearranges the order of attributes, for example.
        # So, we keep a copy of the content for the test.
        content_filename = f"{self.test_content_root}/medium_post_content.txt"
        with open(content_filename, encoding="utf-8") as the_content_file:
            # Many editors and scripts add a final newline, so live with that
            # in our test
            the_content = the_content_file.read()
            assert the_content[-1] == "\n"
            the_content = the_content[:-1]
        self.post_tuple = (
            "A title",
            the_content,
            # slug:
            "2017-04-21-medium-post",
            "2017-04-21 17:11",
            "User Name",
            None,
            (),
            "published",
            "article",
            "html",
        )

    def test_mediumpost2field(self):
        """Parse one post"""
        post_filename = f"{self.test_content_root}/medium_posts/2017-04-21_-medium-post--d1bf01d62ba3.html"
        val = mediumpost2fields(post_filename)
        self.assertEqual(self.post_tuple, val, val)

    def test_mediumposts2field(self):
        """Parse all posts in an export directory"""
        posts = list(mediumposts2fields(f"{self.test_content_root}/medium_posts"))
        self.assertEqual(1, len(posts))
        self.assertEqual(self.post_tuple, posts[0])

    def test_strip_content(self):
        """Strip out unhelpful tags"""
        html_doc = (
            "<section>This keeps <i>lots</i> of <b>tags</b>, but not "
            "the <section>section</section> tags</section>"
        )
        soup = BeautifulSoup(html_doc, "html.parser")
        self.assertEqual(
            "This keeps <i>lots</i> of <b>tags</b>, but not the section tags",
            strip_medium_post_content(soup),
        )

    def test_medium_slug(self):
        # Remove hex stuff at the end
        self.assertEqual(
            "2017-04-27_A-long-title",
            medium_slug(
                "medium-export/posts/2017-04-27_A-long-title--2971442227dd.html"
            ),
        )
        # Remove "--DRAFT" at the end
        self.assertEqual(
            "2017-04-27_A-long-title",
            medium_slug("medium-export/posts/2017-04-27_A-long-title--DRAFT.html"),
        )
        # Remove both (which happens)
        self.assertEqual(
            "draft_How-to-do", medium_slug("draft_How-to-do--DRAFT--87225c81dddd.html")
        )
        # If no hex stuff, leave it alone
        self.assertEqual(
            "2017-04-27_A-long-title",
            medium_slug("medium-export/posts/2017-04-27_A-long-title.html"),
        )



import importlib.metadata
import tarfile
from pathlib import Path
from re import match
from zipfile import ZipFile

import pytest

version = importlib.metadata.version("pelican")


@pytest.mark.skipif(
    "not config.getoption('--check-build')",
    reason="Only run when --check-build is given",
)
def test_wheel_contents(pytestconfig):
    """
    This test should test the contents of the wheel to make sure
    that everything that is needed is included in the final build
    """
    dist_folder = pytestconfig.getoption("--check-build")
    wheels = Path(dist_folder).rglob(f"pelican-{version}-py3-none-any.whl")
    for wheel_file in wheels:
        files_list = ZipFile(wheel_file).namelist()
        # Check if theme files are copied to wheel
        simple_theme = Path("./pelican/themes/simple/templates")
        for x in simple_theme.iterdir():
            assert str(x) in files_list

        # Check if tool templates are copied to wheel
        tools = Path("./pelican/tools/templates")
        for x in tools.iterdir():
            assert str(x) in files_list

        assert "pelican/tools/templates/tasks.py.jinja2" in files_list


@pytest.mark.skipif(
    "not config.getoption('--check-build')",
    reason="Only run when --check-build is given",
)
@pytest.mark.parametrize(
    "expected_file",
    [
        ("THANKS"),
        ("README.rst"),
        ("CONTRIBUTING.rst"),
        ("docs/changelog.rst"),
        ("samples/"),
    ],
)
def test_sdist_contents(pytestconfig, expected_file):
    """
    This test should test the contents of the source distribution to make sure
    that everything that is needed is included in the final build.
    """
    dist_folder = pytestconfig.getoption("--check-build")
    sdist_files = Path(dist_folder).rglob(f"pelican-{version}.tar.gz")
    for dist in sdist_files:
        files_list = tarfile.open(dist, "r:gz").getnames()
        dir_matcher = ""
        if expected_file.endswith("/"):
            dir_matcher = ".*"
        filtered_values = [
            path
            for path in files_list
            if match(rf"^pelican-{version}/{expected_file}{dir_matcher}$", path)
        ]
        assert len(filtered_values) > 0



import copy
import locale
import os
from os.path import abspath, dirname, join

from pelican.settings import (
    DEFAULT_CONFIG,
    DEFAULT_THEME,
    _printf_s_to_format_field,
    configure_settings,
    handle_deprecated_settings,
    read_settings,
)
from pelican.tests.support import unittest


class TestSettingsConfiguration(unittest.TestCase):
    """Provided a file, it should read it, replace the default values,
    append new values to the settings (if any), and apply basic settings
    optimizations.
    """

    def setUp(self):
        self.old_locale = locale.setlocale(locale.LC_ALL)
        locale.setlocale(locale.LC_ALL, "C")
        self.PATH = abspath(dirname(__file__))
        default_conf = join(self.PATH, "default_conf.py")
        self.settings = read_settings(default_conf)

    def tearDown(self):
        locale.setlocale(locale.LC_ALL, self.old_locale)

    def test_overwrite_existing_settings(self):
        self.assertEqual(self.settings.get("SITENAME"), "Alexis' log")
        self.assertEqual(self.settings.get("SITEURL"), "http://blog.notmyidea.org")

    def test_keep_default_settings(self):
        # Keep default settings if not defined.
        self.assertEqual(
            self.settings.get("DEFAULT_CATEGORY"), DEFAULT_CONFIG["DEFAULT_CATEGORY"]
        )

    def test_dont_copy_small_keys(self):
        # Do not copy keys not in caps.
        self.assertNotIn("foobar", self.settings)

    def test_read_empty_settings(self):
        # Ensure an empty settings file results in default settings.
        settings = read_settings(None)
        expected = copy.deepcopy(DEFAULT_CONFIG)
        # Added by configure settings
        expected["FEED_DOMAIN"] = ""
        expected["ARTICLE_EXCLUDES"] = ["pages"]
        expected["PAGE_EXCLUDES"] = [""]
        self.maxDiff = None
        self.assertDictEqual(settings, expected)

    def test_settings_return_independent(self):
        # Make sure that the results from one settings call doesn't
        # effect past or future instances.
        self.PATH = abspath(dirname(__file__))
        default_conf = join(self.PATH, "default_conf.py")
        settings = read_settings(default_conf)
        settings["SITEURL"] = "new-value"
        new_settings = read_settings(default_conf)
        self.assertNotEqual(new_settings["SITEURL"], settings["SITEURL"])

    def test_defaults_not_overwritten(self):
        # This assumes 'SITENAME': 'A Pelican Blog'
        settings = read_settings(None)
        settings["SITENAME"] = "Not a Pelican Blog"
        self.assertNotEqual(settings["SITENAME"], DEFAULT_CONFIG["SITENAME"])

    def test_static_path_settings_safety(self):
        # Disallow static paths from being strings
        settings = {
            "STATIC_PATHS": "foo/bar",
            "THEME_STATIC_PATHS": "bar/baz",
            # These 4 settings are required to run configure_settings
            "PATH": ".",
            "THEME": DEFAULT_THEME,
            "SITEURL": "http://blog.notmyidea.org/",
            "LOCALE": "",
        }
        configure_settings(settings)
        self.assertEqual(settings["STATIC_PATHS"], DEFAULT_CONFIG["STATIC_PATHS"])
        self.assertEqual(
            settings["THEME_STATIC_PATHS"], DEFAULT_CONFIG["THEME_STATIC_PATHS"]
        )

    def test_configure_settings(self):
        # Manipulations to settings should be applied correctly.
        settings = {
            "SITEURL": "http://blog.notmyidea.org/",
            "LOCALE": "",
            "PATH": os.curdir,
            "THEME": DEFAULT_THEME,
        }
        configure_settings(settings)

        # SITEURL should not have a trailing slash
        self.assertEqual(settings["SITEURL"], "http://blog.notmyidea.org")

        # FEED_DOMAIN, if undefined, should default to SITEURL
        self.assertEqual(settings["FEED_DOMAIN"], "http://blog.notmyidea.org")

        settings["FEED_DOMAIN"] = "http://feeds.example.com"
        configure_settings(settings)
        self.assertEqual(settings["FEED_DOMAIN"], "http://feeds.example.com")

    def test_theme_settings_exceptions(self):
        settings = self.settings

        # Check that theme lookup in "pelican/themes" functions as expected
        settings["THEME"] = os.path.split(settings["THEME"])[1]
        configure_settings(settings)
        self.assertEqual(settings["THEME"], DEFAULT_THEME)

        # Check that non-existent theme raises exception
        settings["THEME"] = "foo"
        self.assertRaises(Exception, configure_settings, settings)

    def test_deprecated_dir_setting(self):
        settings = self.settings

        settings["ARTICLE_DIR"] = "foo"
        settings["PAGE_DIR"] = "bar"

        settings = handle_deprecated_settings(settings)

        self.assertEqual(settings["ARTICLE_PATHS"], ["foo"])
        self.assertEqual(settings["PAGE_PATHS"], ["bar"])

        with self.assertRaises(KeyError):
            settings["ARTICLE_DIR"]
            settings["PAGE_DIR"]

    def test_default_encoding(self):
        # Test that the user locale is set if not specified in settings

        locale.setlocale(locale.LC_ALL, "C")
        # empty string = user system locale
        self.assertEqual(self.settings["LOCALE"], [""])

        configure_settings(self.settings)
        lc_time = locale.getlocale(locale.LC_TIME)  # should be set to user locale

        # explicitly set locale to user pref and test
        locale.setlocale(locale.LC_TIME, "")
        self.assertEqual(lc_time, locale.getlocale(locale.LC_TIME))

    def test_invalid_settings_throw_exception(self):
        # Test that the path name is valid

        # test that 'PATH' is set
        settings = {}

        self.assertRaises(Exception, configure_settings, settings)

        # Test that 'PATH' is valid
        settings["PATH"] = ""
        self.assertRaises(Exception, configure_settings, settings)

        # Test nonexistent THEME
        settings["PATH"] = os.curdir
        settings["THEME"] = "foo"

        self.assertRaises(Exception, configure_settings, settings)

    def test__printf_s_to_format_field(self):
        for s in ("%s", "{%s}", "{%s"):
            option = f"foo/{s}/bar.baz"
            result = _printf_s_to_format_field(option, "slug")
            expected = option % "qux"
            found = result.format(slug="qux")
            self.assertEqual(expected, found)

    def test_deprecated_extra_templates_paths(self):
        settings = self.settings
        settings["EXTRA_TEMPLATES_PATHS"] = ["/foo/bar", "/ha"]

        settings = handle_deprecated_settings(settings)

        self.assertEqual(settings["THEME_TEMPLATES_OVERRIDES"], ["/foo/bar", "/ha"])
        self.assertNotIn("EXTRA_TEMPLATES_PATHS", settings)

    def test_deprecated_paginated_direct_templates(self):
        settings = self.settings
        settings["PAGINATED_DIRECT_TEMPLATES"] = ["index", "archives"]
        settings["PAGINATED_TEMPLATES"] = {"index": 10, "category": None}
        settings = handle_deprecated_settings(settings)
        self.assertEqual(
            settings["PAGINATED_TEMPLATES"],
            {"index": 10, "category": None, "archives": None},
        )
        self.assertNotIn("PAGINATED_DIRECT_TEMPLATES", settings)

    def test_deprecated_paginated_direct_templates_from_file(self):
        # This is equivalent to reading a settings file that has
        # PAGINATED_DIRECT_TEMPLATES defined but no PAGINATED_TEMPLATES.
        settings = read_settings(
            None, override={"PAGINATED_DIRECT_TEMPLATES": ["index", "archives"]}
        )
        self.assertEqual(
            settings["PAGINATED_TEMPLATES"],
            {
                "archives": None,
                "author": None,
                "index": None,
                "category": None,
                "tag": None,
            },
        )
        self.assertNotIn("PAGINATED_DIRECT_TEMPLATES", settings)

    def test_theme_and_extra_templates_exception(self):
        settings = self.settings
        settings["EXTRA_TEMPLATES_PATHS"] = ["/ha"]
        settings["THEME_TEMPLATES_OVERRIDES"] = ["/foo/bar"]

        self.assertRaises(Exception, handle_deprecated_settings, settings)

    def test_slug_and_slug_regex_substitutions_exception(self):
        settings = {}
        settings["SLUG_REGEX_SUBSTITUTIONS"] = [("C++", "cpp")]
        settings["TAG_SUBSTITUTIONS"] = [("C#", "csharp")]

        self.assertRaises(Exception, handle_deprecated_settings, settings)

    def test_deprecated_slug_substitutions(self):
        default_slug_regex_subs = self.settings["SLUG_REGEX_SUBSTITUTIONS"]

        # If no deprecated setting is set, don't set new ones
        settings = {}
        settings = handle_deprecated_settings(settings)
        self.assertNotIn("SLUG_REGEX_SUBSTITUTIONS", settings)
        self.assertNotIn("TAG_REGEX_SUBSTITUTIONS", settings)
        self.assertNotIn("CATEGORY_REGEX_SUBSTITUTIONS", settings)
        self.assertNotIn("AUTHOR_REGEX_SUBSTITUTIONS", settings)

        # If SLUG_SUBSTITUTIONS is set, set {SLUG, AUTHOR}_REGEX_SUBSTITUTIONS
        # correctly, don't set {CATEGORY, TAG}_REGEX_SUBSTITUTIONS
        settings = {}
        settings["SLUG_SUBSTITUTIONS"] = [("C++", "cpp")]
        settings = handle_deprecated_settings(settings)
        self.assertEqual(
            settings.get("SLUG_REGEX_SUBSTITUTIONS"),
            [(r"C\+\+", "cpp")] + default_slug_regex_subs,
        )
        self.assertNotIn("TAG_REGEX_SUBSTITUTIONS", settings)
        self.assertNotIn("CATEGORY_REGEX_SUBSTITUTIONS", settings)
        self.assertEqual(
            settings.get("AUTHOR_REGEX_SUBSTITUTIONS"), default_slug_regex_subs
        )

        # If {CATEGORY, TAG, AUTHOR}_SUBSTITUTIONS are set, set
        # {CATEGORY, TAG, AUTHOR}_REGEX_SUBSTITUTIONS correctly, don't set
        # SLUG_REGEX_SUBSTITUTIONS
        settings = {}
        settings["TAG_SUBSTITUTIONS"] = [("C#", "csharp")]
        settings["CATEGORY_SUBSTITUTIONS"] = [("C#", "csharp")]
        settings["AUTHOR_SUBSTITUTIONS"] = [("Alexander Todorov", "atodorov")]
        settings = handle_deprecated_settings(settings)
        self.assertNotIn("SLUG_REGEX_SUBSTITUTIONS", settings)
        self.assertEqual(
            settings["TAG_REGEX_SUBSTITUTIONS"],
            [(r"C\#", "csharp")] + default_slug_regex_subs,
        )
        self.assertEqual(
            settings["CATEGORY_REGEX_SUBSTITUTIONS"],
            [(r"C\#", "csharp")] + default_slug_regex_subs,
        )
        self.assertEqual(
            settings["AUTHOR_REGEX_SUBSTITUTIONS"],
            [(r"Alexander\ Todorov", "atodorov")] + default_slug_regex_subs,
        )

        # If {SLUG, CATEGORY, TAG, AUTHOR}_SUBSTITUTIONS are set, set
        # {SLUG, CATEGORY, TAG, AUTHOR}_REGEX_SUBSTITUTIONS correctly
        settings = {}
        settings["SLUG_SUBSTITUTIONS"] = [("C++", "cpp")]
        settings["TAG_SUBSTITUTIONS"] = [("C#", "csharp")]
        settings["CATEGORY_SUBSTITUTIONS"] = [("C#", "csharp")]
        settings["AUTHOR_SUBSTITUTIONS"] = [("Alexander Todorov", "atodorov")]
        settings = handle_deprecated_settings(settings)
        self.assertEqual(
            settings["TAG_REGEX_SUBSTITUTIONS"],
            [(r"C\+\+", "cpp")] + [(r"C\#", "csharp")] + default_slug_regex_subs,
        )
        self.assertEqual(
            settings["CATEGORY_REGEX_SUBSTITUTIONS"],
            [(r"C\+\+", "cpp")] + [(r"C\#", "csharp")] + default_slug_regex_subs,
        )
        self.assertEqual(
            settings["AUTHOR_REGEX_SUBSTITUTIONS"],
            [(r"Alexander\ Todorov", "atodorov")] + default_slug_regex_subs,
        )

        # Handle old 'skip' flags correctly
        settings = {}
        settings["SLUG_SUBSTITUTIONS"] = [("C++", "cpp", True)]
        settings["AUTHOR_SUBSTITUTIONS"] = [("Alexander Todorov", "atodorov", False)]
        settings = handle_deprecated_settings(settings)
        self.assertEqual(
            settings.get("SLUG_REGEX_SUBSTITUTIONS"),
            [(r"C\+\+", "cpp")] + [(r"(?u)\A\s*", ""), (r"(?u)\s*\Z", "")],
        )
        self.assertEqual(
            settings["AUTHOR_REGEX_SUBSTITUTIONS"],
            [(r"Alexander\ Todorov", "atodorov")] + default_slug_regex_subs,
        )

    def test_deprecated_slug_substitutions_from_file(self):
        # This is equivalent to reading a settings file that has
        # SLUG_SUBSTITUTIONS defined but no SLUG_REGEX_SUBSTITUTIONS.
        settings = read_settings(
            None, override={"SLUG_SUBSTITUTIONS": [("C++", "cpp")]}
        )
        self.assertEqual(
            settings["SLUG_REGEX_SUBSTITUTIONS"],
            [(r"C\+\+", "cpp")] + self.settings["SLUG_REGEX_SUBSTITUTIONS"],
        )
        self.assertNotIn("SLUG_SUBSTITUTIONS", settings)



import os
from unittest.mock import PropertyMock, patch

from pelican import readers
from pelican.tests.support import get_settings, unittest
from pelican.utils import SafeDatetime

CUR_DIR = os.path.dirname(__file__)
CONTENT_PATH = os.path.join(CUR_DIR, "content")


def _path(*args):
    return os.path.join(CONTENT_PATH, *args)


class ReaderTest(unittest.TestCase):
    def read_file(self, path, **kwargs):
        # Isolate from future API changes to readers.read_file

        r = readers.Readers(settings=get_settings(**kwargs))
        return r.read_file(base_path=CONTENT_PATH, path=path)

    def assertDictHasSubset(self, dictionary, subset):
        for key, value in subset.items():
            if key in dictionary:
                real_value = dictionary.get(key)
                self.assertEqual(
                    value,
                    real_value,
                    f"Expected {key} to have value {value}, but was {real_value}",
                )
            else:
                self.fail(f"Expected {key} to have value {value}, but was not in Dict")

    def test_markdown_disabled(self):
        with patch.object(
            readers.MarkdownReader, "enabled", new_callable=PropertyMock
        ) as attr_mock:
            attr_mock.return_value = False
            readrs = readers.Readers(settings=get_settings())
            self.assertEqual(
                set(readers.MarkdownReader.file_extensions),
                readrs.disabled_readers.keys(),
            )
            for val in readrs.disabled_readers.values():
                self.assertEqual(readers.MarkdownReader, val.__class__)


class TestAssertDictHasSubset(ReaderTest):
    def setUp(self):
        self.dictionary = {"key-a": "val-a", "key-b": "val-b"}

    def tearDown(self):
        self.dictionary = None

    def test_subset(self):
        self.assertDictHasSubset(self.dictionary, {"key-a": "val-a"})

    def test_equal(self):
        self.assertDictHasSubset(self.dictionary, self.dictionary)

    def test_fail_not_set(self):
        self.assertRaisesRegex(
            AssertionError,
            r"Expected.*key-c.*to have value.*val-c.*but was not in Dict",
            self.assertDictHasSubset,
            self.dictionary,
            {"key-c": "val-c"},
        )

    def test_fail_wrong_val(self):
        self.assertRaisesRegex(
            AssertionError,
            r"Expected .*key-a.* to have value .*val-b.* but was .*val-a.*",
            self.assertDictHasSubset,
            self.dictionary,
            {"key-a": "val-b"},
        )


class DefaultReaderTest(ReaderTest):
    def test_readfile_unknown_extension(self):
        with self.assertRaises(TypeError):
            self.read_file(path="article_with_metadata.unknownextension")

    def test_readfile_path_metadata_implicit_dates(self):
        test_file = "article_with_metadata_implicit_dates.html"
        page = self.read_file(path=test_file, DEFAULT_DATE="fs")
        expected = {
            "date": SafeDatetime.fromtimestamp(os.stat(_path(test_file)).st_mtime),
            "modified": SafeDatetime.fromtimestamp(os.stat(_path(test_file)).st_mtime),
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_readfile_path_metadata_explicit_dates(self):
        test_file = "article_with_metadata_explicit_dates.html"
        page = self.read_file(path=test_file, DEFAULT_DATE="fs")
        expected = {
            "date": SafeDatetime(2010, 12, 2, 10, 14),
            "modified": SafeDatetime(2010, 12, 31, 23, 59),
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_readfile_path_metadata_implicit_date_explicit_modified(self):
        test_file = "article_with_metadata_implicit_date_explicit_modified.html"
        page = self.read_file(path=test_file, DEFAULT_DATE="fs")
        expected = {
            "date": SafeDatetime.fromtimestamp(os.stat(_path(test_file)).st_mtime),
            "modified": SafeDatetime(2010, 12, 2, 10, 14),
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_readfile_path_metadata_explicit_date_implicit_modified(self):
        test_file = "article_with_metadata_explicit_date_implicit_modified.html"
        page = self.read_file(path=test_file, DEFAULT_DATE="fs")
        expected = {
            "date": SafeDatetime(2010, 12, 2, 10, 14),
            "modified": SafeDatetime.fromtimestamp(os.stat(_path(test_file)).st_mtime),
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_find_empty_alt(self):
        with patch("pelican.readers.logger") as log_mock:
            content = [
                '<img alt="" src="test-image.png" width="300px" />',
                '<img src="test-image.png"  width="300px" alt="" />',
            ]

            for tag in content:
                readers.find_empty_alt(tag, "/test/path")
                log_mock.warning.assert_called_with(
                    "Empty alt attribute for image %s in %s",
                    "test-image.png",
                    "/test/path",
                    extra={"limit_msg": "Other images have empty alt attributes"},
                )


class RstReaderTest(ReaderTest):
    def test_article_with_metadata(self):
        page = self.read_file(path="article_with_metadata.rst")
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "This is a super article !",
            "summary": '<p class="first last">Multi-line metadata should be'
            " supported\nas well as <strong>inline"
            " markup</strong> and stuff to &quot;typogrify"
            "&quot;...</p>\n",
            "date": SafeDatetime(2010, 12, 2, 10, 14),
            "modified": SafeDatetime(2010, 12, 2, 10, 20),
            "tags": ["foo", "bar", "foobar"],
            "custom_field": "http://notmyidea.org",
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_capitalized_metadata(self):
        page = self.read_file(path="article_with_capitalized_metadata.rst")
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "This is a super article !",
            "summary": '<p class="first last">Multi-line metadata should be'
            " supported\nas well as <strong>inline"
            " markup</strong> and stuff to &quot;typogrify"
            "&quot;...</p>\n",
            "date": SafeDatetime(2010, 12, 2, 10, 14),
            "modified": SafeDatetime(2010, 12, 2, 10, 20),
            "tags": ["foo", "bar", "foobar"],
            "custom_field": "http://notmyidea.org",
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_filename_metadata(self):
        page = self.read_file(
            path="2012-11-29_rst_w_filename_meta#foo-bar.rst", FILENAME_METADATA=None
        )
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "Rst with filename metadata",
            "reader": "rst",
        }
        self.assertDictHasSubset(page.metadata, expected)

        page = self.read_file(
            path="2012-11-29_rst_w_filename_meta#foo-bar.rst",
            FILENAME_METADATA=r"(?P<date>\d{4}-\d{2}-\d{2}).*",
        )
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "Rst with filename metadata",
            "date": SafeDatetime(2012, 11, 29),
            "reader": "rst",
        }
        self.assertDictHasSubset(page.metadata, expected)

        page = self.read_file(
            path="2012-11-29_rst_w_filename_meta#foo-bar.rst",
            FILENAME_METADATA=(
                r"(?P<date>\d{4}-\d{2}-\d{2})"
                r"_(?P<Slug>.*)"
                r"#(?P<MyMeta>.*)-(?P<author>.*)"
            ),
        )
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "Rst with filename metadata",
            "date": SafeDatetime(2012, 11, 29),
            "slug": "rst_w_filename_meta",
            "mymeta": "foo",
            "reader": "rst",
        }
        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_optional_filename_metadata(self):
        page = self.read_file(
            path="2012-11-29_rst_w_filename_meta#foo-bar.rst",
            FILENAME_METADATA=r"(?P<date>\d{4}-\d{2}-\d{2})?",
        )
        expected = {
            "date": SafeDatetime(2012, 11, 29),
            "reader": "rst",
        }
        self.assertDictHasSubset(page.metadata, expected)

        page = self.read_file(
            path="article.rst", FILENAME_METADATA=r"(?P<date>\d{4}-\d{2}-\d{2})?"
        )
        expected = {
            "reader": "rst",
        }
        self.assertDictHasSubset(page.metadata, expected)
        self.assertNotIn("date", page.metadata, "Date should not be set.")

    def test_article_metadata_key_lowercase(self):
        # Keys of metadata should be lowercase.
        reader = readers.RstReader(settings=get_settings())
        content, metadata = reader.read(_path("article_with_uppercase_metadata.rst"))

        self.assertIn("category", metadata, "Key should be lowercase.")
        self.assertEqual("Yeah", metadata.get("category"), "Value keeps case.")

    def test_article_extra_path_metadata(self):
        input_with_metadata = "2012-11-29_rst_w_filename_meta#foo-bar.rst"
        page_metadata = self.read_file(
            path=input_with_metadata,
            FILENAME_METADATA=(
                r"(?P<date>\d{4}-\d{2}-\d{2})"
                r"_(?P<Slug>.*)"
                r"#(?P<MyMeta>.*)-(?P<author>.*)"
            ),
            EXTRA_PATH_METADATA={
                input_with_metadata: {"key-1a": "value-1a", "key-1b": "value-1b"}
            },
        )
        expected_metadata = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "Rst with filename metadata",
            "date": SafeDatetime(2012, 11, 29),
            "slug": "rst_w_filename_meta",
            "mymeta": "foo",
            "reader": "rst",
            "key-1a": "value-1a",
            "key-1b": "value-1b",
        }
        self.assertDictHasSubset(page_metadata.metadata, expected_metadata)

        input_file_path_without_metadata = "article.rst"
        page_without_metadata = self.read_file(
            path=input_file_path_without_metadata,
            EXTRA_PATH_METADATA={
                input_file_path_without_metadata: {"author": "Charlès Overwrite"}
            },
        )
        expected_without_metadata = {
            "category": "misc",
            "author": "Charlès Overwrite",
            "title": "Article title",
            "reader": "rst",
        }
        self.assertDictHasSubset(
            page_without_metadata.metadata, expected_without_metadata
        )

    def test_article_extra_path_metadata_dont_overwrite(self):
        # EXTRA_PATH_METADATA['author'] should get ignored
        # since we don't overwrite already set values
        input_file_path = "2012-11-29_rst_w_filename_meta#foo-bar.rst"
        page = self.read_file(
            path=input_file_path,
            FILENAME_METADATA=(
                r"(?P<date>\d{4}-\d{2}-\d{2})"
                r"_(?P<Slug>.*)"
                r"#(?P<MyMeta>.*)-(?P<orginalauthor>.*)"
            ),
            EXTRA_PATH_METADATA={
                input_file_path: {"author": "Charlès Overwrite", "key-1b": "value-1b"}
            },
        )
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "Rst with filename metadata",
            "date": SafeDatetime(2012, 11, 29),
            "slug": "rst_w_filename_meta",
            "mymeta": "foo",
            "reader": "rst",
            "key-1b": "value-1b",
        }
        self.assertDictHasSubset(page.metadata, expected)

    def test_article_extra_path_metadata_recurse(self):
        parent = "TestCategory"
        notparent = "TestCategory/article"
        path = "TestCategory/article_without_category.rst"

        epm = {
            parent: {
                "epmr_inherit": parent,
                "epmr_override": parent,
            },
            notparent: {"epmr_bogus": notparent},
            path: {
                "epmr_override": path,
            },
        }
        expected_metadata = {
            "epmr_inherit": parent,
            "epmr_override": path,
        }

        page = self.read_file(path=path, EXTRA_PATH_METADATA=epm)
        self.assertDictHasSubset(page.metadata, expected_metadata)

        # Make sure vars aren't getting "inherited" by mistake...
        path = "article.rst"
        page = self.read_file(path=path, EXTRA_PATH_METADATA=epm)
        for k in expected_metadata.keys():
            self.assertNotIn(k, page.metadata)

        # Same, but for edge cases where one file's name is a prefix of
        # another.
        path = "TestCategory/article_without_category.rst"
        page = self.read_file(path=path, EXTRA_PATH_METADATA=epm)
        for k in epm[notparent].keys():
            self.assertNotIn(k, page.metadata)

    def test_typogrify(self):
        # if nothing is specified in the settings, the content should be
        # unmodified
        page = self.read_file(path="article.rst")
        expected = (
            "<p>THIS is some content. With some stuff to "
            "&quot;typogrify&quot;...</p>\n<p>Now with added "
            'support for <abbr title="three letter acronym">'
            "TLA</abbr>.</p>\n"
        )

        self.assertEqual(page.content, expected)

        try:
            # otherwise, typogrify should be applied
            page = self.read_file(path="article.rst", TYPOGRIFY=True)
            expected = (
                '<p><span class="caps">THIS</span> is some content. '
                "With some stuff to&nbsp;&#8220;typogrify&#8221;&#8230;</p>\n"
                '<p>Now with added support for <abbr title="three letter '
                'acronym"><span class="caps">TLA</span></abbr>.</p>\n'
            )

            self.assertEqual(page.content, expected)
        except ImportError:
            return unittest.skip("need the typogrify distribution")

    def test_typogrify_summary(self):
        # if nothing is specified in the settings, the summary should be
        # unmodified
        page = self.read_file(path="article_with_metadata.rst")
        expected = (
            '<p class="first last">Multi-line metadata should be'
            " supported\nas well as <strong>inline"
            " markup</strong> and stuff to &quot;typogrify"
            "&quot;...</p>\n"
        )

        self.assertEqual(page.metadata["summary"], expected)

        try:
            # otherwise, typogrify should be applied
            page = self.read_file(path="article_with_metadata.rst", TYPOGRIFY=True)
            expected = (
                '<p class="first last">Multi-line metadata should be'
                " supported\nas well as <strong>inline"
                " markup</strong> and stuff to&nbsp;&#8220;typogrify"
                "&#8221;&#8230;</p>\n"
            )

            self.assertEqual(page.metadata["summary"], expected)
        except ImportError:
            return unittest.skip("need the typogrify distribution")

    def test_typogrify_ignore_tags(self):
        try:
            # typogrify should be able to ignore user specified tags,
            # but tries to be clever with widont extension
            page = self.read_file(
                path="article.rst", TYPOGRIFY=True, TYPOGRIFY_IGNORE_TAGS=["p"]
            )
            expected = (
                "<p>THIS is some content. With some stuff to&nbsp;"
                "&quot;typogrify&quot;...</p>\n<p>Now with added "
                'support for <abbr title="three letter acronym">'
                "TLA</abbr>.</p>\n"
            )

            self.assertEqual(page.content, expected)

            # typogrify should ignore code blocks by default because
            # code blocks are composed inside the pre tag
            page = self.read_file(path="article_with_code_block.rst", TYPOGRIFY=True)

            expected = (
                "<p>An article with some&nbsp;code</p>\n"
                '<div class="highlight"><pre><span></span>'
                '<span class="n">x</span>'
                ' <span class="o">&amp;</span>'
                ' <span class="n">y</span>\n</pre></div>\n'
                "<p>A block&nbsp;quote:</p>\n<blockquote>\nx "
                '<span class="amp">&amp;</span> y</blockquote>\n'
                "<p>Normal:\nx"
                ' <span class="amp">&amp;</span>'
                "&nbsp;y"
                "</p>\n"
            )

            self.assertEqual(page.content, expected)

            # instruct typogrify to also ignore blockquotes
            page = self.read_file(
                path="article_with_code_block.rst",
                TYPOGRIFY=True,
                TYPOGRIFY_IGNORE_TAGS=["blockquote"],
            )

            expected = (
                "<p>An article with some&nbsp;code</p>\n"
                '<div class="highlight"><pre><span>'
                '</span><span class="n">x</span>'
                ' <span class="o">&amp;</span>'
                ' <span class="n">y</span>\n</pre></div>\n'
                "<p>A block&nbsp;quote:</p>\n<blockquote>\nx "
                "&amp; y</blockquote>\n"
                "<p>Normal:\nx"
                ' <span class="amp">&amp;</span>'
                "&nbsp;y"
                "</p>\n"
            )

            self.assertEqual(page.content, expected)
        except ImportError:
            return unittest.skip("need the typogrify distribution")
        except TypeError:
            return unittest.skip("need typogrify version 2.0.4 or later")

    def test_article_with_multiple_authors(self):
        page = self.read_file(path="article_with_multiple_authors.rst")
        expected = {"authors": ["First Author", "Second Author"]}

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_multiple_authors_semicolon(self):
        page = self.read_file(path="article_with_multiple_authors_semicolon.rst")
        expected = {"authors": ["Author, First", "Author, Second"]}

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_multiple_authors_list(self):
        page = self.read_file(path="article_with_multiple_authors_list.rst")
        expected = {"authors": ["Author, First", "Author, Second"]}

        self.assertDictHasSubset(page.metadata, expected)

    def test_default_date_formats(self):
        tuple_date = self.read_file(path="article.rst", DEFAULT_DATE=(2012, 5, 1))
        string_date = self.read_file(path="article.rst", DEFAULT_DATE="2012-05-01")

        self.assertEqual(tuple_date.metadata["date"], string_date.metadata["date"])

    def test_parse_error(self):
        # Verify that it raises an Exception, not nothing and not SystemExit or
        # some such
        with self.assertRaisesRegex(Exception, "underline too short"):
            self.read_file(path="../parse_error/parse_error.rst")

    def test_typogrify_dashes_config(self):
        # Test default config
        page = self.read_file(
            path="article_with_typogrify_dashes.rst",
            TYPOGRIFY=True,
            TYPOGRIFY_DASHES="default",
        )
        expected = "<p>One: -; Two: &#8212;; Three:&nbsp;&#8212;-</p>\n"
        expected_title = "One -, two &#8212;, three &#8212;-&nbsp;dashes!"

        self.assertEqual(page.content, expected)
        self.assertEqual(page.title, expected_title)

        # Test 'oldschool' variant
        page = self.read_file(
            path="article_with_typogrify_dashes.rst",
            TYPOGRIFY=True,
            TYPOGRIFY_DASHES="oldschool",
        )
        expected = "<p>One: -; Two: &#8211;; Three:&nbsp;&#8212;</p>\n"
        expected_title = "One -, two &#8211;, three &#8212;&nbsp;dashes!"

        self.assertEqual(page.content, expected)
        self.assertEqual(page.title, expected_title)

        # Test 'oldschool_inverted' variant
        page = self.read_file(
            path="article_with_typogrify_dashes.rst",
            TYPOGRIFY=True,
            TYPOGRIFY_DASHES="oldschool_inverted",
        )
        expected = "<p>One: -; Two: &#8212;; Three:&nbsp;&#8211;</p>\n"
        expected_title = "One -, two &#8212;, three &#8211;&nbsp;dashes!"

        self.assertEqual(page.content, expected)
        self.assertEqual(page.title, expected_title)


@unittest.skipUnless(readers.Markdown, "markdown isn't installed")
class MdReaderTest(ReaderTest):
    def test_article_with_metadata(self):
        reader = readers.MarkdownReader(settings=get_settings())
        content, metadata = reader.read(_path("article_with_md_extension.md"))
        expected = {
            "category": "test",
            "title": "Test md File",
            "summary": "<p>I have a lot to test</p>",
            "date": SafeDatetime(2010, 12, 2, 10, 14),
            "modified": SafeDatetime(2010, 12, 2, 10, 20),
            "tags": ["foo", "bar", "foobar"],
        }
        self.assertDictHasSubset(metadata, expected)

        content, metadata = reader.read(
            _path("article_with_markdown_and_nonascii_summary.md")
        )
        expected = {
            "title": "マックOS X 10.8でパイソンとVirtualenvをインストールと設定",
            "summary": "<p>パイソンとVirtualenvをまっくでインストールする方法について明確に説明します。</p>",
            "category": "指導書",
            "date": SafeDatetime(2012, 12, 20),
            "modified": SafeDatetime(2012, 12, 22),
            "tags": ["パイソン", "マック"],
            "slug": "python-virtualenv-on-mac-osx-mountain-lion-10.8",
        }
        self.assertDictHasSubset(metadata, expected)

    def test_article_with_footnote(self):
        settings = get_settings()
        ec = settings["MARKDOWN"]["extension_configs"]
        ec["markdown.extensions.footnotes"] = {"SEPARATOR": "-"}
        reader = readers.MarkdownReader(settings)
        content, metadata = reader.read(_path("article_with_markdown_and_footnote.md"))
        expected_content = (
            "<p>This is some content"
            '<sup id="fnref-1"><a class="footnote-ref" href="#fn-1"'
            ">1</a></sup>"
            " with some footnotes"
            '<sup id="fnref-footnote"><a class="footnote-ref" '
            'href="#fn-footnote">2</a></sup></p>\n'
            '<div class="footnote">\n'
            '<hr>\n<ol>\n<li id="fn-1">\n'
            "<p>Numbered footnote&#160;"
            '<a class="footnote-backref" href="#fnref-1" '
            'title="Jump back to footnote 1 in the text">&#8617;</a></p>\n'
            '</li>\n<li id="fn-footnote">\n'
            "<p>Named footnote&#160;"
            '<a class="footnote-backref" href="#fnref-footnote"'
            ' title="Jump back to footnote 2 in the text">&#8617;</a></p>\n'
            "</li>\n</ol>\n</div>"
        )
        expected_metadata = {
            "title": "Article with markdown containing footnotes",
            "summary": (
                "<p>Summary with <strong>inline</strong> markup "
                "<em>should</em> be supported.</p>"
            ),
            "date": SafeDatetime(2012, 10, 31),
            "modified": SafeDatetime(2012, 11, 1),
            "multiline": [
                "Line Metadata should be handle properly.",
                "See syntax of Meta-Data extension of Python Markdown package:",
                "If a line is indented by 4 or more spaces,",
                "that line is assumed to be an additional line of the value",
                "for the previous keyword.",
                "A keyword may have as many lines as desired.",
            ],
        }
        self.assertEqual(content, expected_content)
        self.assertDictHasSubset(metadata, expected_metadata)

    def test_article_with_file_extensions(self):
        reader = readers.MarkdownReader(settings=get_settings())
        # test to ensure the md file extension is being processed by the
        # correct reader
        content, metadata = reader.read(_path("article_with_md_extension.md"))
        expected = (
            "<h1>Test Markdown File Header</h1>\n"
            "<h2>Used for pelican test</h2>\n"
            "<p>The quick brown fox jumped over the lazy dog's back.</p>"
        )
        self.assertEqual(content, expected)
        # test to ensure the mkd file extension is being processed by the
        # correct reader
        content, metadata = reader.read(_path("article_with_mkd_extension.mkd"))
        expected = (
            "<h1>Test Markdown File Header</h1>\n<h2>Used for pelican"
            " test</h2>\n<p>This is another markdown test file.  Uses"
            " the mkd extension.</p>"
        )
        self.assertEqual(content, expected)
        # test to ensure the markdown file extension is being processed by the
        # correct reader
        content, metadata = reader.read(
            _path("article_with_markdown_extension.markdown")
        )
        expected = (
            "<h1>Test Markdown File Header</h1>\n<h2>Used for pelican"
            " test</h2>\n<p>This is another markdown test file.  Uses"
            " the markdown extension.</p>"
        )
        self.assertEqual(content, expected)
        # test to ensure the mdown file extension is being processed by the
        # correct reader
        content, metadata = reader.read(_path("article_with_mdown_extension.mdown"))
        expected = (
            "<h1>Test Markdown File Header</h1>\n<h2>Used for pelican"
            " test</h2>\n<p>This is another markdown test file.  Uses"
            " the mdown extension.</p>"
        )
        self.assertEqual(content, expected)

    def test_article_with_markdown_markup_extension(self):
        # test to ensure the markdown markup extension is being processed as
        # expected
        page = self.read_file(
            path="article_with_markdown_markup_extensions.md",
            MARKDOWN={
                "extension_configs": {
                    "markdown.extensions.toc": {},
                    "markdown.extensions.codehilite": {},
                    "markdown.extensions.extra": {},
                }
            },
        )
        expected = (
            '<div class="toc">\n'
            "<ul>\n"
            '<li><a href="#level1">Level1</a><ul>\n'
            '<li><a href="#level2">Level2</a></li>\n'
            "</ul>\n"
            "</li>\n"
            "</ul>\n"
            "</div>\n"
            '<h2 id="level1">Level1</h2>\n'
            '<h3 id="level2">Level2</h3>'
        )

        self.assertEqual(page.content, expected)

    def test_article_with_filename_metadata(self):
        page = self.read_file(
            path="2012-11-30_md_w_filename_meta#foo-bar.md", FILENAME_METADATA=None
        )
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
        }
        self.assertDictHasSubset(page.metadata, expected)

        page = self.read_file(
            path="2012-11-30_md_w_filename_meta#foo-bar.md",
            FILENAME_METADATA=r"(?P<date>\d{4}-\d{2}-\d{2}).*",
        )
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "date": SafeDatetime(2012, 11, 30),
        }
        self.assertDictHasSubset(page.metadata, expected)

        page = self.read_file(
            path="2012-11-30_md_w_filename_meta#foo-bar.md",
            FILENAME_METADATA=(
                r"(?P<date>\d{4}-\d{2}-\d{2})"
                r"_(?P<Slug>.*)"
                r"#(?P<MyMeta>.*)-(?P<author>.*)"
            ),
        )
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "date": SafeDatetime(2012, 11, 30),
            "slug": "md_w_filename_meta",
            "mymeta": "foo",
        }
        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_optional_filename_metadata(self):
        page = self.read_file(
            path="2012-11-30_md_w_filename_meta#foo-bar.md",
            FILENAME_METADATA=r"(?P<date>\d{4}-\d{2}-\d{2})?",
        )
        expected = {
            "date": SafeDatetime(2012, 11, 30),
            "reader": "markdown",
        }
        self.assertDictHasSubset(page.metadata, expected)

        page = self.read_file(
            path="empty.md", FILENAME_METADATA=r"(?P<date>\d{4}-\d{2}-\d{2})?"
        )
        expected = {
            "reader": "markdown",
        }
        self.assertDictHasSubset(page.metadata, expected)
        self.assertNotIn("date", page.metadata, "Date should not be set.")

    def test_duplicate_tags_or_authors_are_removed(self):
        reader = readers.MarkdownReader(settings=get_settings())
        content, metadata = reader.read(_path("article_with_duplicate_tags_authors.md"))
        expected = {
            "tags": ["foo", "bar", "foobar"],
            "authors": ["Author, First", "Author, Second"],
        }
        self.assertDictHasSubset(metadata, expected)

    def test_metadata_not_parsed_for_metadata(self):
        settings = get_settings()
        settings["FORMATTED_FIELDS"] = ["summary"]

        reader = readers.MarkdownReader(settings=settings)
        content, metadata = reader.read(
            _path("article_with_markdown_and_nested_metadata.md")
        )
        expected = {
            "title": "Article with markdown and nested summary metadata",
            "summary": "<p>Test: This metadata value looks like metadata</p>",
        }
        self.assertDictHasSubset(metadata, expected)

    def test_empty_file(self):
        reader = readers.MarkdownReader(settings=get_settings())
        content, metadata = reader.read(_path("empty.md"))

        self.assertEqual(metadata, {})
        self.assertEqual(content, "")

    def test_empty_file_with_bom(self):
        reader = readers.MarkdownReader(settings=get_settings())
        content, metadata = reader.read(_path("empty_with_bom.md"))

        self.assertEqual(metadata, {})
        self.assertEqual(content, "")

    def test_typogrify_dashes_config(self):
        # Test default config
        page = self.read_file(
            path="article_with_typogrify_dashes.md",
            TYPOGRIFY=True,
            TYPOGRIFY_DASHES="default",
        )
        expected = "<p>One: -; Two: &#8212;; Three:&nbsp;&#8212;-</p>"
        expected_title = "One -, two &#8212;, three &#8212;-&nbsp;dashes!"

        self.assertEqual(page.content, expected)
        self.assertEqual(page.title, expected_title)

        # Test 'oldschool' variant
        page = self.read_file(
            path="article_with_typogrify_dashes.md",
            TYPOGRIFY=True,
            TYPOGRIFY_DASHES="oldschool",
        )
        expected = "<p>One: -; Two: &#8211;; Three:&nbsp;&#8212;</p>"
        expected_title = "One -, two &#8211;, three &#8212;&nbsp;dashes!"

        self.assertEqual(page.content, expected)
        self.assertEqual(page.title, expected_title)

        # Test 'oldschool_inverted' variant
        page = self.read_file(
            path="article_with_typogrify_dashes.md",
            TYPOGRIFY=True,
            TYPOGRIFY_DASHES="oldschool_inverted",
        )
        expected = "<p>One: -; Two: &#8212;; Three:&nbsp;&#8211;</p>"
        expected_title = "One -, two &#8212;, three &#8211;&nbsp;dashes!"

        self.assertEqual(page.content, expected)
        self.assertEqual(page.title, expected_title)

    def test_metadata_has_no_discarded_data(self):
        md_filename = "article_with_markdown_and_empty_tags.md"

        r = readers.Readers(
            cache_name="cache", settings=get_settings(CACHE_CONTENT=True)
        )
        page = r.read_file(base_path=CONTENT_PATH, path=md_filename)

        __, cached_metadata = r.get_cached_data(_path(md_filename), (None, None))

        expected = {"title": "Article with markdown and empty tags"}
        self.assertEqual(cached_metadata, expected)
        self.assertNotIn("tags", page.metadata)
        self.assertDictHasSubset(page.metadata, expected)


class HTMLReaderTest(ReaderTest):
    def test_article_with_comments(self):
        page = self.read_file(path="article_with_comments.html")

        self.assertEqual(
            """
        Body content
        <!--  This comment is included (including extra whitespace)   -->
    """,
            page.content,
        )

    def test_article_with_keywords(self):
        page = self.read_file(path="article_with_keywords.html")
        expected = {
            "tags": ["foo", "bar", "foobar"],
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_metadata(self):
        page = self.read_file(path="article_with_metadata.html")
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "This is a super article !",
            "summary": "Summary and stuff",
            "date": SafeDatetime(2010, 12, 2, 10, 14),
            "tags": ["foo", "bar", "foobar"],
            "custom_field": "http://notmyidea.org",
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_multiple_similar_metadata_tags(self):
        page = self.read_file(path="article_with_multiple_metadata_tags.html")
        expected = {
            "custom_field": ["https://getpelican.com", "https://www.eff.org"],
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_multiple_authors(self):
        page = self.read_file(path="article_with_multiple_authors.html")
        expected = {"authors": ["First Author", "Second Author"]}

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_metadata_and_contents_attrib(self):
        page = self.read_file(path="article_with_metadata_and_contents.html")
        expected = {
            "category": "yeah",
            "author": "Alexis Métaireau",
            "title": "This is a super article !",
            "summary": "Summary and stuff",
            "date": SafeDatetime(2010, 12, 2, 10, 14),
            "tags": ["foo", "bar", "foobar"],
            "custom_field": "http://notmyidea.org",
        }
        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_null_attributes(self):
        page = self.read_file(path="article_with_null_attributes.html")

        self.assertEqual(
            """
        Ensure that empty attributes are copied properly.
        <input name="test" disabled style="" />
    """,
            page.content,
        )

    def test_article_with_attributes_containing_double_quotes(self):
        page = self.read_file(
            path="article_with_attributes_containing_" + "double_quotes.html"
        )
        self.assertEqual(
            """
        Ensure that if an attribute value contains a double quote, it is
        surrounded with single quotes, otherwise with double quotes.
        <span data-test="'single quoted string'">Span content</span>
        <span data-test='"double quoted string"'>Span content</span>
        <span data-test="string without quotes">Span content</span>
    """,
            page.content,
        )

    def test_article_metadata_key_lowercase(self):
        # Keys of metadata should be lowercase.
        page = self.read_file(path="article_with_uppercase_metadata.html")

        # Key should be lowercase
        self.assertIn("category", page.metadata, "Key should be lowercase.")

        # Value should keep cases
        self.assertEqual("Yeah", page.metadata.get("category"))

    def test_article_with_nonconformant_meta_tags(self):
        page = self.read_file(path="article_with_nonconformant_meta_tags.html")
        expected = {
            "summary": "Summary and stuff",
            "title": "Article with Nonconformant HTML meta tags",
        }

        self.assertDictHasSubset(page.metadata, expected)

    def test_article_with_inline_svg(self):
        page = self.read_file(path="article_with_inline_svg.html")
        expected = {
            "title": "Article with an inline SVG",
        }
        self.assertDictHasSubset(page.metadata, expected)



#!/usr/bin/env python

import argparse
import locale
import os
from typing import Mapping

from jinja2 import Environment, FileSystemLoader

try:
    import zoneinfo
except ModuleNotFoundError:
    from backports import zoneinfo

try:
    import readline  # NOQA
except ImportError:
    pass

try:
    import tzlocal

    if hasattr(tzlocal.get_localzone(), "zone"):
        _DEFAULT_TIMEZONE = tzlocal.get_localzone().zone
    else:
        _DEFAULT_TIMEZONE = tzlocal.get_localzone_name()
except ModuleNotFoundError:
    _DEFAULT_TIMEZONE = "Europe/Rome"

from pelican import __version__

locale.setlocale(locale.LC_ALL, "")
try:
    _DEFAULT_LANGUAGE = locale.getlocale()[0]
except ValueError:
    # Don't fail on macosx: "unknown locale: UTF-8"
    _DEFAULT_LANGUAGE = None
if _DEFAULT_LANGUAGE is None:
    _DEFAULT_LANGUAGE = "en"
else:
    _DEFAULT_LANGUAGE = _DEFAULT_LANGUAGE.split("_")[0]

_TEMPLATES_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "templates")
_jinja_env = Environment(
    loader=FileSystemLoader(_TEMPLATES_DIR),
    trim_blocks=True,
    keep_trailing_newline=True,
)


_GITHUB_PAGES_BRANCHES = {"personal": "main", "project": "gh-pages"}

CONF = {
    "pelican": "pelican",
    "pelicanopts": "",
    "basedir": os.curdir,
    "ftp_host": "localhost",
    "ftp_user": "anonymous",
    "ftp_target_dir": "/",
    "ssh_host": "localhost",
    "ssh_port": 22,
    "ssh_user": "root",
    "ssh_target_dir": "/var/www",
    "s3_bucket": "my_s3_bucket",
    "cloudfiles_username": "my_rackspace_username",
    "cloudfiles_api_key": "my_rackspace_api_key",
    "cloudfiles_container": "my_cloudfiles_container",
    "dropbox_dir": "~/Dropbox/Public/",
    "github_pages_branch": _GITHUB_PAGES_BRANCHES["project"],
    "default_pagination": 10,
    "siteurl": "",
    "lang": _DEFAULT_LANGUAGE,
    "timezone": _DEFAULT_TIMEZONE,
}

# url for list of valid timezones
_TZ_URL = "https://en.wikipedia.org/wiki/List_of_tz_database_time_zones"


# Create a 'marked' default path, to determine if someone has supplied
# a path on the command-line.
class _DEFAULT_PATH_TYPE(str):  # noqa: SLOT000
    is_default_path = True


_DEFAULT_PATH = _DEFAULT_PATH_TYPE(os.curdir)


def ask(question, answer=str, default=None, length=None):
    if answer is str:
        r = ""
        while True:
            if default:
                r = input(f"> {question} [{default}] ")
            else:
                r = input(f"> {question} ")

            r = r.strip()

            if len(r) <= 0:
                if default:
                    r = default
                    break
                else:
                    print("You must enter something")
            elif length and len(r) != length:
                print(f"Entry must be {length} characters long")
            else:
                break

        return r

    elif answer is bool:
        r = None
        while True:
            if default is True:
                r = input(f"> {question} (Y/n) ")
            elif default is False:
                r = input(f"> {question} (y/N) ")
            else:
                r = input(f"> {question} (y/n) ")

            r = r.strip().lower()

            if r in ("y", "yes"):
                r = True
                break
            elif r in ("n", "no"):
                r = False
                break
            elif not r:
                r = default
                break
            else:
                print("You must answer 'yes' or 'no'")
        return r
    elif answer is int:
        r = None
        while True:
            if default:
                r = input(f"> {question} [{default}] ")
            else:
                r = input(f"> {question} ")

            r = r.strip()

            if not r:
                r = default
                break

            try:
                r = int(r)
                break
            except ValueError:
                print("You must enter an integer")
        return r
    else:
        raise NotImplementedError("Argument `answer` must be str, bool, or integer")


def ask_timezone(question, default, tzurl):
    """Prompt for time zone and validate input"""
    tz_dict = {tz.lower(): tz for tz in zoneinfo.available_timezones()}
    while True:
        r = ask(question, str, default)
        r = r.strip().replace(" ", "_").lower()
        if r in tz_dict.keys():
            r = tz_dict[r]
            break
        else:
            print(f"Please enter a valid time zone:\n (check [{tzurl}])")
    return r


def render_jinja_template(tmpl_name: str, tmpl_vars: Mapping, target_path: str):
    try:
        with open(
            os.path.join(CONF["basedir"], target_path), "w", encoding="utf-8"
        ) as fd:
            _template = _jinja_env.get_template(tmpl_name)
            fd.write(_template.render(**tmpl_vars))
    except OSError as e:
        print(f"Error: {e}")


def main():
    parser = argparse.ArgumentParser(
        description="A kickstarter for Pelican",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-p", "--path", default=_DEFAULT_PATH, help="The path to generate the blog into"
    )
    parser.add_argument(
        "-t", "--title", metavar="title", help="Set the title of the website"
    )
    parser.add_argument(
        "-a", "--author", metavar="author", help="Set the author name of the website"
    )
    parser.add_argument(
        "-l", "--lang", metavar="lang", help="Set the default web site language"
    )

    args = parser.parse_args()

    print(
        f"""Welcome to pelican-quickstart v{__version__}.

This script will help you create a new Pelican-based website.

Please answer the following questions so this script can generate the files
needed by Pelican.

    """
    )

    project = os.path.join(os.environ.get("VIRTUAL_ENV", os.curdir), ".project")
    no_path_was_specified = hasattr(args.path, "is_default_path")
    if os.path.isfile(project) and no_path_was_specified:
        CONF["basedir"] = open(project).read().rstrip("\n")
        print(
            "Using project associated with current virtual environment. "
            "Will save to:\n{}\n".format(CONF["basedir"])
        )
    else:
        CONF["basedir"] = os.path.abspath(
            os.path.expanduser(
                ask(
                    "Where do you want to create your new web site?",
                    answer=str,
                    default=args.path,
                )
            )
        )

    CONF["sitename"] = ask(
        "What will be the title of this web site?", answer=str, default=args.title
    )
    CONF["author"] = ask(
        "Who will be the author of this web site?", answer=str, default=args.author
    )
    CONF["lang"] = ask(
        "What will be the default language of this web site?",
        str,
        args.lang or CONF["lang"],
        2,
    )

    if ask(
        "Do you want to specify a URL prefix? e.g., https://example.com  ",
        answer=bool,
        default=True,
    ):
        CONF["siteurl"] = ask(
            "What is your URL prefix? (see above example; no trailing slash)",
            str,
            CONF["siteurl"],
        )

    CONF["with_pagination"] = ask(
        "Do you want to enable article pagination?",
        bool,
        bool(CONF["default_pagination"]),
    )

    if CONF["with_pagination"]:
        CONF["default_pagination"] = ask(
            "How many articles per page do you want?",
            int,
            CONF["default_pagination"],
        )
    else:
        CONF["default_pagination"] = False

    CONF["timezone"] = ask_timezone(
        "What is your time zone?", CONF["timezone"], _TZ_URL
    )

    automation = ask(
        "Do you want to generate a tasks.py/Makefile "
        "to automate generation and publishing?",
        bool,
        True,
    )

    if automation:
        if ask(
            "Do you want to upload your website using FTP?", answer=bool, default=False
        ):
            CONF["ftp"] = (True,)
            CONF["ftp_host"] = ask(
                "What is the hostname of your FTP server?", str, CONF["ftp_host"]
            )
            CONF["ftp_user"] = ask(
                "What is your username on that server?", str, CONF["ftp_user"]
            )
            CONF["ftp_target_dir"] = ask(
                "Where do you want to put your web site on that server?",
                str,
                CONF["ftp_target_dir"],
            )
        if ask(
            "Do you want to upload your website using SSH?", answer=bool, default=False
        ):
            CONF["ssh"] = (True,)
            CONF["ssh_host"] = ask(
                "What is the hostname of your SSH server?", str, CONF["ssh_host"]
            )
            CONF["ssh_port"] = ask(
                "What is the port of your SSH server?", int, CONF["ssh_port"]
            )
            CONF["ssh_user"] = ask(
                "What is your username on that server?", str, CONF["ssh_user"]
            )
            CONF["ssh_target_dir"] = ask(
                "Where do you want to put your web site on that server?",
                str,
                CONF["ssh_target_dir"],
            )

        if ask(
            "Do you want to upload your website using Dropbox?",
            answer=bool,
            default=False,
        ):
            CONF["dropbox"] = (True,)
            CONF["dropbox_dir"] = ask(
                "Where is your Dropbox directory?", str, CONF["dropbox_dir"]
            )

        if ask(
            "Do you want to upload your website using S3?", answer=bool, default=False
        ):
            CONF["s3"] = (True,)
            CONF["s3_bucket"] = ask(
                "What is the name of your S3 bucket?", str, CONF["s3_bucket"]
            )

        if ask(
            "Do you want to upload your website using Rackspace Cloud Files?",
            answer=bool,
            default=False,
        ):
            CONF["cloudfiles"] = (True,)
            CONF["cloudfiles_username"] = ask(
                "What is your Rackspace Cloud username?",
                str,
                CONF["cloudfiles_username"],
            )
            CONF["cloudfiles_api_key"] = ask(
                "What is your Rackspace Cloud API key?",
                str,
                CONF["cloudfiles_api_key"],
            )
            CONF["cloudfiles_container"] = ask(
                "What is the name of your Cloud Files container?",
                str,
                CONF["cloudfiles_container"],
            )

        if ask(
            "Do you want to upload your website using GitHub Pages?",
            answer=bool,
            default=False,
        ):
            CONF["github"] = (True,)
            if ask(
                "Is this your personal page (username.github.io)?",
                answer=bool,
                default=False,
            ):
                CONF["github_pages_branch"] = _GITHUB_PAGES_BRANCHES["personal"]
            else:
                CONF["github_pages_branch"] = _GITHUB_PAGES_BRANCHES["project"]

    try:
        os.makedirs(os.path.join(CONF["basedir"], "content"))
    except OSError as e:
        print(f"Error: {e}")

    try:
        os.makedirs(os.path.join(CONF["basedir"], "output"))
    except OSError as e:
        print(f"Error: {e}")

    conf_python = {}
    for key, value in CONF.items():
        conf_python[key] = repr(value)
    render_jinja_template("pelicanconf.py.jinja2", conf_python, "pelicanconf.py")

    render_jinja_template("publishconf.py.jinja2", CONF, "publishconf.py")

    if automation:
        render_jinja_template("tasks.py.jinja2", CONF, "tasks.py")
        render_jinja_template("Makefile.jinja2", CONF, "Makefile")

    print("Done. Your new project is available at {}".format(CONF["basedir"]))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import os
import shutil
import sys


def err(msg, die=None):
    """Print an error message and exits if an exit code is given"""
    sys.stderr.write(msg + "\n")
    if die:
        sys.exit(die if isinstance(die, int) else 1)


try:
    import pelican
except ImportError:
    err(
        "Cannot import pelican.\nYou must "
        "install Pelican in order to run this script.",
        -1,
    )


_THEMES_PATH = os.path.join(
    os.path.dirname(os.path.abspath(pelican.__file__)), "themes"
)

__version__ = "0.2"
_BUILTIN_THEMES = ["simple", "notmyidea"]


def main():
    """Main function"""

    parser = argparse.ArgumentParser(description="""Install themes for Pelican""")

    excl = parser.add_mutually_exclusive_group()
    excl.add_argument(
        "-l",
        "--list",
        dest="action",
        action="store_const",
        const="list",
        help="Show the themes already installed and exit",
    )
    excl.add_argument(
        "-p",
        "--path",
        dest="action",
        action="store_const",
        const="path",
        help="Show the themes path and exit",
    )
    excl.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"pelican-themes v{__version__}",
        help="Print the version of this script",
    )

    parser.add_argument(
        "-i",
        "--install",
        dest="to_install",
        nargs="+",
        metavar="theme path",
        help="The themes to install",
    )
    parser.add_argument(
        "-r",
        "--remove",
        dest="to_remove",
        nargs="+",
        metavar="theme name",
        help="The themes to remove",
    )
    parser.add_argument(
        "-U",
        "--upgrade",
        dest="to_upgrade",
        nargs="+",
        metavar="theme path",
        help="The themes to upgrade",
    )
    parser.add_argument(
        "-s",
        "--symlink",
        dest="to_symlink",
        nargs="+",
        metavar="theme path",
        help="Same as `--install', but create a symbolic link instead of "
        "copying the theme. Useful for theme development",
    )
    parser.add_argument(
        "-c",
        "--clean",
        dest="clean",
        action="store_true",
        help="Remove the broken symbolic links of the theme path",
    )

    parser.add_argument(
        "-v", "--verbose", dest="verbose", action="store_true", help="Verbose output"
    )

    args = parser.parse_args()

    to_install = args.to_install or args.to_upgrade
    to_sym = args.to_symlink or args.clean

    if args.action:
        if args.action == "list":
            list_themes(args.verbose)
        elif args.action == "path":
            print(_THEMES_PATH)
    elif to_install or args.to_remove or to_sym:
        if args.to_remove:
            if args.verbose:
                print("Removing themes...")

            for i in args.to_remove:
                remove(i, v=args.verbose)

        if args.to_install:
            if args.verbose:
                print("Installing themes...")

            for i in args.to_install:
                install(i, v=args.verbose)

        if args.to_upgrade:
            if args.verbose:
                print("Upgrading themes...")

            for i in args.to_upgrade:
                install(i, v=args.verbose, u=True)

        if args.to_symlink:
            if args.verbose:
                print("Linking themes...")

            for i in args.to_symlink:
                symlink(i, v=args.verbose)

        if args.clean:
            if args.verbose:
                print("Cleaning the themes directory...")

            clean(v=args.verbose)
    else:
        print("No argument given... exiting.")


def themes():
    """Returns the list of the themes"""
    for i in os.listdir(_THEMES_PATH):
        e = os.path.join(_THEMES_PATH, i)

        if os.path.isdir(e):
            if os.path.islink(e):
                yield (e, os.readlink(e))
            else:
                yield (e, None)


def list_themes(v=False):
    """Display the list of the themes"""
    for theme_path, link_target in themes():
        if not v:
            theme_path = os.path.basename(theme_path)
        if link_target:
            if v:
                print(theme_path + (" (symbolic link to `" + link_target + "')"))
            else:
                print(theme_path + "@")
        else:
            print(theme_path)


def remove(theme_name, v=False):
    """Removes a theme"""

    theme_name = theme_name.replace("/", "")
    target = os.path.join(_THEMES_PATH, theme_name)

    if theme_name in _BUILTIN_THEMES:
        err(
            theme_name + " is a builtin theme.\n"
            "You cannot remove a builtin theme with this script, "
            "remove it by hand if you want."
        )
    elif os.path.islink(target):
        if v:
            print("Removing link `" + target + "'")
        os.remove(target)
    elif os.path.isdir(target):
        if v:
            print("Removing directory `" + target + "'")
        shutil.rmtree(target)
    elif os.path.exists(target):
        err(target + " : not a valid theme")
    else:
        err(target + " : no such file or directory")


def install(path, v=False, u=False):
    """Installs a theme"""
    if not os.path.exists(path):
        err(path + " : no such file or directory")
    elif not os.path.isdir(path):
        err(path + " : not a directory")
    else:
        theme_name = os.path.basename(os.path.normpath(path))
        theme_path = os.path.join(_THEMES_PATH, theme_name)
        exists = os.path.exists(theme_path)
        if exists and not u:
            err(path + " : already exists")
        elif exists:
            remove(theme_name, v)
            install(path, v)
        else:
            if v:
                print(f"Copying '{path}' to '{theme_path}' ...")
            try:
                shutil.copytree(path, theme_path)

                try:
                    if os.name == "posix":
                        for root, dirs, files in os.walk(theme_path):
                            for d in dirs:
                                dname = os.path.join(root, d)
                                os.chmod(dname, 493)  # 0o755
                            for f in files:
                                fname = os.path.join(root, f)
                                os.chmod(fname, 420)  # 0o644
                except OSError as e:
                    err(
                        "Cannot change permissions of files "
                        f"or directory in `{theme_path}':\n{e!s}",
                        die=False,
                    )
            except Exception as e:
                err(f"Cannot copy `{path}' to `{theme_path}':\n{e!s}")


def symlink(path, v=False):
    """Symbolically link a theme"""
    path = os.path.realpath(path)
    if not os.path.exists(path):
        err(path + " : no such file or directory")
    elif not os.path.isdir(path):
        err(path + " : not a directory")
    else:
        theme_name = os.path.basename(os.path.normpath(path))
        theme_path = os.path.join(_THEMES_PATH, theme_name)
        if os.path.exists(theme_path):
            err(path + " : already exists")
        else:
            if v:
                print(f"Linking `{path}' to `{theme_path}' ...")
            try:
                os.symlink(path, theme_path)
            except Exception as e:
                err(f"Cannot link `{path}' to `{theme_path}':\n{e!s}")


def is_broken_link(path):
    """Returns True if the path given as is a broken symlink"""
    path = os.path.realpath(path)
    return not os.path.exists(path)


def clean(v=False):
    """Removes the broken symbolic links"""
    c = 0
    for path in os.listdir(_THEMES_PATH):
        path = os.path.join(_THEMES_PATH, path)
        if os.path.islink(path) and is_broken_link(path):
            if v:
                print(f"Removing {path}")
            try:
                os.remove(path)
            except OSError:
                print(f"Error: cannot remove {path}")
            else:
                c += 1

    print(f"\nRemoved {c} broken links")



import os
from io import BytesIO
from shutil import rmtree
from tempfile import mkdtemp

from pelican.server import ComplexHTTPRequestHandler
from pelican.tests.support import unittest


class MockRequest:
    def makefile(self, *args, **kwargs):
        return BytesIO(b"")


class MockServer:
    pass


class TestServer(unittest.TestCase):
    def setUp(self):
        self.server = MockServer()
        self.temp_output = mkdtemp(prefix="pelicantests.")
        self.old_cwd = os.getcwd()
        os.chdir(self.temp_output)

    def tearDown(self):
        os.chdir(self.old_cwd)
        rmtree(self.temp_output)

    def test_get_path_that_exists(self):
        handler = ComplexHTTPRequestHandler(
            MockRequest(), ("0.0.0.0", 8888), self.server
        )
        handler.base_path = self.temp_output

        open(os.path.join(self.temp_output, "foo.html"), "a").close()
        os.mkdir(os.path.join(self.temp_output, "foo"))
        open(os.path.join(self.temp_output, "foo", "index.html"), "a").close()

        os.mkdir(os.path.join(self.temp_output, "bar"))
        open(os.path.join(self.temp_output, "bar", "index.html"), "a").close()

        os.mkdir(os.path.join(self.temp_output, "baz"))

        for suffix in ["", "/"]:
            # foo.html has precedence over foo/index.html
            path = handler.get_path_that_exists("foo" + suffix)
            self.assertEqual(path, "foo.html")

            # folder with index.html should return folder/index.html
            path = handler.get_path_that_exists("bar" + suffix)
            self.assertEqual(path, "bar/index.html")

            # folder without index.html should return same as input
            path = handler.get_path_that_exists("baz" + suffix)
            self.assertEqual(path, "baz" + suffix)

            # not existing path should return None
            path = handler.get_path_that_exists("quux" + suffix)
            self.assertIsNone(path)



import re

from docutils import nodes, utils
from docutils.parsers.rst import Directive, directives, roles
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import TextLexer, get_lexer_by_name

import pelican.settings as pys


class Pygments(Directive):
    """Source code syntax highlighting."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        "anchorlinenos": directives.flag,
        "classprefix": directives.unchanged,
        "hl_lines": directives.unchanged,
        "lineanchors": directives.unchanged,
        "linenos": directives.unchanged,
        "linenospecial": directives.nonnegative_int,
        "linenostart": directives.nonnegative_int,
        "linenostep": directives.nonnegative_int,
        "lineseparator": directives.unchanged,
        "linespans": directives.unchanged,
        "nobackground": directives.flag,
        "nowrap": directives.flag,
        "tagsfile": directives.unchanged,
        "tagurlformat": directives.unchanged,
    }
    has_content = True

    def run(self):
        self.assert_has_content()
        try:
            lexer = get_lexer_by_name(self.arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = TextLexer()

        # Fetch the defaults
        if pys.PYGMENTS_RST_OPTIONS is not None:
            for k, v in pys.PYGMENTS_RST_OPTIONS.items():
                # Locally set options overrides the defaults
                if k not in self.options:
                    self.options[k] = v

        if "linenos" in self.options and self.options["linenos"] not in (
            "table",
            "inline",
        ):
            if self.options["linenos"] == "none":
                self.options.pop("linenos")
            else:
                self.options["linenos"] = "table"

        for flag in ("nowrap", "nobackground", "anchorlinenos"):
            if flag in self.options:
                self.options[flag] = True

        # noclasses should already default to False, but just in case...
        formatter = HtmlFormatter(noclasses=False, **self.options)
        parsed = highlight("\n".join(self.content), lexer, formatter)
        return [nodes.raw("", parsed, format="html")]


directives.register_directive("code-block", Pygments)
directives.register_directive("sourcecode", Pygments)


_abbr_re = re.compile(r"\((.*)\)$", re.DOTALL)


class abbreviation(nodes.Inline, nodes.TextElement):
    pass


def abbr_role(typ, rawtext, text, lineno, inliner, options=None, content=None):
    text = utils.unescape(text)
    m = _abbr_re.search(text)
    if m is None:
        return [abbreviation(text, text)], []
    abbr = text[: m.start()].strip()
    expl = m.group(1)
    return [abbreviation(abbr, abbr, explanation=expl)], []


roles.register_local_role("abbr", abbr_role)



from unittest.mock import Mock

from pelican.tests.support import unittest


class Test_abbr_role(unittest.TestCase):
    def call_it(self, text):
        from pelican.rstdirectives import abbr_role

        rawtext = text
        lineno = 42
        inliner = Mock(name="inliner")
        nodes, system_messages = abbr_role("abbr", rawtext, text, lineno, inliner)
        self.assertEqual(system_messages, [])
        self.assertEqual(len(nodes), 1)
        return nodes[0]

    def test(self):
        node = self.call_it("Abbr (Abbreviation)")
        self.assertEqual(node.astext(), "Abbr")
        self.assertEqual(node["explanation"], "Abbreviation")

    def test_newlines_in_explanation(self):
        node = self.call_it("CUL (See you\nlater)")
        self.assertEqual(node.astext(), "CUL")
        self.assertEqual(node["explanation"], "See you\nlater")

    def test_newlines_in_abbr(self):
        node = self.call_it("US of\nA \n (USA)")
        self.assertEqual(node.astext(), "US of\nA")
        self.assertEqual(node["explanation"], "USA")
